@@ -1,25 +1,26 @@
+# -*- coding: utf-8 -*-
 #############################################################################
 # File          : FilesCheck.py
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Mon Oct  4 19:32:49 1999
-# Version       : $Id: FilesCheck.py 1379 2007-09-13 17:39:11Z scop $
+# Version       : $Id: FilesCheck.py 1564 2009-03-08 16:08:12Z scop $
 # Purpose       : test various aspects on files: locations, owner, groups,
 #                 permission, setuid, setgid...
 #############################################################################
 
-from Filter import *
-import AbstractCheck
-from Pkg import is_utf8, is_utf8_str
-import rpm
+import os
 import re
 import stat
 import string
-import os
-try:
-    from textwrap import fill # python >= 2.3
-except ImportError:
-    def fill(text, width=70): return text
+
+import rpm
+
+from Filter import addDetails, printError, printWarning
+from Pkg import is_utf8, is_utf8_str
+import AbstractCheck
+import Config
+
 
 # must be kept in sync with the filesystem package
 STANDARD_DIRS = (
@@ -153,7 +154,7 @@ STANDARD_DIRS = (
     '/var/tmp',
     )
 
-DEFAULT_GAMES_GROUPS='Games'
+DEFAULT_GAMES_GROUPS = 'Games'
 
 DEFAULT_DANGLING_EXCEPTIONS = (['consolehelper$', 'usermode-consoleonly'],
                                )
@@ -166,57 +167,59 @@ DEFAULT_STANDARD_GROUPS = ('root', 'bin', 'daemon', 'adm', 'lp', 'sync',
                            'shutdown', 'halt', 'mail', 'news', 'uucp',
                            'man', 'nobody',)
 
-tmp_regex=re.compile('^/tmp/|^(/var|/usr)/tmp/')
-sub_bin_regex=re.compile('^(/usr)?/s?bin/\S+/')
-backup_regex=re.compile('~$|\#[^/]+\#$')
-compr_regex=re.compile('\.(gz|z|Z|zip|bz2)$')
-absolute_regex=re.compile('^/([^/]+)')
-absolute2_regex=re.compile('^/?([^/]+)')
-points_regex=re.compile('^\.\./(.*)')
-doc_regex=re.compile('^/usr(/share|/X11R6)?/(doc|man|info)/')
-bin_regex=re.compile('^(/usr)?/s?bin/')
+tmp_regex = re.compile('^/tmp/|^(/var|/usr)/tmp/')
+sub_bin_regex = re.compile('^(/usr)?/s?bin/\S+/')
+backup_regex = re.compile('~$|\#[^/]+\#$')
+compr_regex = re.compile('\.(gz|z|Z|zip|bz2|lzma)$')
+absolute_regex = re.compile('^/([^/]+)')
+absolute2_regex = re.compile('^/?([^/]+)')
+points_regex = re.compile('^\.\./(.*)')
+doc_regex = re.compile('^/usr(/share|/X11R6)?/(doc|man|info)/')
+bin_regex = re.compile('^(/usr)?/s?bin/')
 includefile_regex = re.compile('\.(c|h)(pp|xx)?$', re.IGNORECASE)
 develfile_regex = re.compile('\.(a|cmxa?|mli?)$')
-buildconfigfile_regex=re.compile('(\.pc|/bin/.+-config)$')
-sofile_regex=re.compile('/lib(64)?/[^/]+\.so$')
-devel_regex=re.compile('(.*)-(debug(info)?|devel|source|static)$')
-debuginfo_package_regex=re.compile('-debug(info)?$')
-lib_regex=re.compile('lib(64)?/lib[^/]*\.so\..*')
-ldconfig_regex=re.compile('^[^#]*ldconfig', re.MULTILINE)
-depmod_regex=re.compile('^[^#]*depmod', re.MULTILINE)
-install_info_regex=re.compile('^[^#]*install-info', re.MULTILINE)
-perl_temp_file=re.compile('.*perl.*/(\.packlist|perllocal\.pod)$')
-scm_regex=re.compile('/CVS/[^/]+$|/\.(cvs|git|hg)ignore$|/\.hgtags$|/\.(git|hg|svn)/|/(\.arch-ids|{arch})/')
-htaccess_regex=re.compile('\.htaccess$')
-games_path_regex=re.compile('^/usr(/lib(64)?)?/games/')
-games_group_regex=re.compile(Config.getOption('RpmGamesGroups', DEFAULT_GAMES_GROUPS))
-source_regex=re.compile('\.(c|cc|cpp|ui)$')
-dangling_exceptions=Config.getOption('DanglingSymlinkExceptions', DEFAULT_DANGLING_EXCEPTIONS)
-logrotate_regex=re.compile('^/etc/logrotate\.d/(.*)')
-module_rpms_ok=Config.getOption('KernelModuleRPMsOK', 1)
-kernel_modules_regex=re.compile('^/lib/modules/(2\.[23456]\.[0-9]+[^/]*?)/')
-kernel_package_regex=re.compile('^kernel(22)?(-)?(smp|enterprise|bigmem|secure|BOOT|i686-up-4GB|p3-smp-64GB)?')
-normal_zero_length_regex=re.compile('^/etc/security/console\.apps/|/\.nosearch$|/__init__\.py$')
-perl_regex=re.compile('^/usr/lib/perl5/(?:vendor_perl/)?([0-9]+\.[0-9]+)\.([0-9]+)/')
-python_regex=re.compile('^/usr/lib/python([.0-9]+)/')
-perl_version_trick=Config.getOption('PerlVersionTrick', 1)
-log_regex=re.compile('^/var/log/[^/]+$')
-lib_path_regex=re.compile('^(/usr(/X11R6)?)?/lib(64)?')
-lib_package_regex=re.compile('^(lib|.+-libs)')
-hidden_file_regex=re.compile('/\.[^/]*$')
-mispelled_macro_regex=re.compile('%{.*}')
-siteperl_perl_regex=re.compile('/site_perl/')
-manifest_perl_regex=re.compile('^/usr/share/doc/perl-.*/MANIFEST(\.SKIP)?$');
-shebang_regex=re.compile('^#!\s*(\S*)')
-interpreter_regex=re.compile('^/(usr/)?s?bin/[^/]+$')
-script_regex=re.compile('^/((usr/)?s?bin|etc/(rc\.d/init\.d|X11/xinit\.d|cron\.(hourly|daily|monthly|weekly)))/')
-sourced_script_regex=re.compile('^/etc/(bash_completion\.d|profile\.d)/')
-use_utf8=Config.getOption('UseUTF8', Config.USEUTF8_DEFAULT)
-meta_package_re=re.compile(Config.getOption('MetaPackageRegexp', '^(bundle|task)-'))
+buildconfigfile_regex = re.compile('(\.pc|/bin/.+-config)$')
+# room for improvement with catching more -R, but also for false positives...
+buildconfig_rpath_regex = re.compile('(?:-rpath|Wl,-R)\\b')
+sofile_regex = re.compile('/lib(64)?/(.+/)?lib[^/]+\.so$')
+devel_regex = re.compile('(.*)-(debug(info)?|devel|headers|source|static)$')
+debuginfo_package_regex = re.compile('-debug(info)?$')
+lib_regex = re.compile('lib(64)?/lib[^/]*\.so\..*')
+ldconfig_regex = re.compile('^[^#]*ldconfig', re.MULTILINE)
+depmod_regex = re.compile('^[^#]*depmod', re.MULTILINE)
+install_info_regex = re.compile('^[^#]*install-info', re.MULTILINE)
+perl_temp_file_regex = re.compile('.*perl.*/(\.packlist|perllocal\.pod)$')
+scm_regex = re.compile('/CVS/[^/]+$|/\.(bzr|cvs|git|hg)ignore$|/\.hgtags$|/\.(bzr|git|hg|svn)/|/(\.arch-ids|{arch})/')
+games_path_regex = re.compile('^/usr(/lib(64)?)?/games/')
+games_group_regex = re.compile(Config.getOption('RpmGamesGroups', DEFAULT_GAMES_GROUPS))
+dangling_exceptions = Config.getOption('DanglingSymlinkExceptions', DEFAULT_DANGLING_EXCEPTIONS)
+logrotate_regex = re.compile('^/etc/logrotate\.d/(.*)')
+module_rpms_ok = Config.getOption('KernelModuleRPMsOK', 1)
+kernel_modules_regex = re.compile('^/lib/modules/(2\.[23456]\.[0-9]+[^/]*?)/')
+kernel_package_regex = re.compile('^kernel(22)?(-)?(smp|enterprise|bigmem|secure|BOOT|i686-up-4GB|p3-smp-64GB)?')
+normal_zero_length_regex = re.compile('^/etc/security/console\.apps/|/\.nosearch$|/__init__\.py$')
+perl_regex = re.compile('^/usr/lib/perl5/(?:vendor_perl/)?([0-9]+\.[0-9]+)\.([0-9]+)/')
+python_regex = re.compile('^/usr/lib/python([.0-9]+)/')
+perl_version_trick = Config.getOption('PerlVersionTrick', 1)
+log_regex = re.compile('^/var/log/[^/]+$')
+lib_path_regex = re.compile('^(/usr(/X11R6)?)?/lib(64)?')
+lib_package_regex = re.compile('^(lib|.+-libs)')
+hidden_file_regex = re.compile('/\.[^/]*$')
+mispelled_macro_regex = re.compile('%{.*}')
+siteperl_perl_regex = re.compile('/site_perl/')
+manifest_perl_regex = re.compile('^/usr/share/doc/perl-.*/MANIFEST(\.SKIP)?$')
+shebang_regex = re.compile('^#!\s*(\S*)')
+interpreter_regex = re.compile('^/(usr/)?(s?bin|games|libexec(/.+)?|(lib(64)?|share)/.+)/[^/]+$')
+script_regex = re.compile('^/((usr/)?s?bin|etc/(rc\.d/init\.d|X11/xinit\.d|cron\.(hourly|daily|monthly|weekly)))/')
+sourced_script_regex = re.compile('^/etc/(bash_completion\.d|profile\.d)/')
+use_utf8 = Config.getOption('UseUTF8', Config.USEUTF8_DEFAULT)
+skipdocs_regex = re.compile(Config.getOption('SkipDocsRegexp', '\.(?:rtf|x?html?|ml[ily]?)$'), re.IGNORECASE)
+meta_package_regex = re.compile(Config.getOption('MetaPackageRegexp', '^(bundle|task)-'))
 filesys_packages = ['filesystem'] # TODO: make configurable?
+quotes_regex = re.compile('[\'"]+')
 
 for idx in range(0, len(dangling_exceptions)):
-    dangling_exceptions[idx][0]=re.compile(dangling_exceptions[idx][0])
+    dangling_exceptions[idx][0] = re.compile(dangling_exceptions[idx][0])
 del idx
 
 use_relative_symlinks = Config.getOption("UseRelativeSymlinks", 1)
@@ -280,47 +283,52 @@ class FilesCheck(AbstractCheck.AbstractCheck):
             return
 
         # Check if the package is a development package
-        devel_pkg=devel_regex.search(pkg.name)
+        devel_pkg = devel_regex.search(pkg.name)
 
-        config_files=pkg.configFiles()
-        ghost_files=pkg.ghostFiles()
-        doc_files=pkg.docFiles()
-        req_names=pkg.req_names()
-        lib_package=lib_package_regex.search(pkg.name)
-        is_kernel_package=kernel_package_regex.search(pkg.name)
+        config_files = pkg.configFiles()
+        ghost_files = pkg.ghostFiles()
+        doc_files = pkg.docFiles()
+        req_names = pkg.req_names()
+        lib_package = lib_package_regex.search(pkg.name)
+        is_kernel_package = kernel_package_regex.search(pkg.name)
 
         # report these errors only once
-        perl_dep_error=0
-        python_dep_error=0
-        lib_file=0
-        non_lib_file=0
-        log_file=0
-        logrotate_file=0
+        perl_dep_error = 0
+        python_dep_error = 0
+        lib_file = 0
+        non_lib_file = 0
+        log_file = 0
+        logrotate_file = 0
 
         if not doc_files:
             printWarning(pkg, 'no-documentation')
 
-        if len(files.keys()) and meta_package_re.search(pkg.name):
-            printWarning(pkg, 'file-in-meta-package')
-
-        if debuginfo_package_regex.search(pkg.name) and not files:
+        if files:
+            if meta_package_regex.search(pkg.name):
+                printWarning(pkg, 'file-in-meta-package')
+        elif debuginfo_package_regex.search(pkg.name):
             printError(pkg, 'empty-debuginfo-package')
 
-        for f in files.keys():
-            enreg=files[f]
-            mode=enreg[0]
-            user=enreg[1]
-            group=enreg[2]
-            size=enreg[4]
+        # Unique (rdev, inode) combinations
+        hardlinks = {} 
+
+        for f, pkgfile in files.items():
+            mode = pkgfile.mode
+            user = pkgfile.user
+            group = pkgfile.group
+            link = pkgfile.linkto
+            size = pkgfile.size
+            rdev = pkgfile.rdev
+            inode = pkgfile.inode
             is_doc = f in doc_files
             nonexec_file = 0
 
             if mispelled_macro_regex.search(f):
                 printWarning(pkg, 'mispelled-macro', f)
             if standard_users and not user in standard_users:
-                printError(pkg, 'non-standard-uid', f, user)
+                printWarning(pkg, 'non-standard-uid', f, user)
             if standard_groups and not group in standard_groups:
-                printError(pkg, 'non-standard-gid', f, group)
+                printWarning(pkg, 'non-standard-gid', f, group)
 
             if not module_rpms_ok and kernel_modules_regex.search(f) and not is_kernel_package:
                 printError(pkg, "kernel-modules-not-in-kernel-packages", f)
@@ -345,7 +353,7 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                 printError(pkg, 'version-control-internal-file', f)
             elif f.endswith('/.htaccess'):
                 printError(pkg, 'htaccess-file', f)
-            elif hidden_file_regex.search(f):
+            elif hidden_file_regex.search(f) and not f.startswith("/etc/skel/"):
                 printWarning(pkg, 'hidden-file-or-dir', f)
             elif manifest_perl_regex.search(f):
                 printWarning(pkg, 'manifest-in-perl-module', f)
@@ -354,31 +362,30 @@ class FilesCheck(AbstractCheck.AbstractCheck):
             elif f == '/usr/info/dir' or f == '/usr/share/info/dir':
                 printError(pkg, 'info-dir-file', f)
 
-            res=logrotate_regex.search(f)
-            logrotate_file=res or logrotate_file
-            if res and res.group(1) != pkg.name:
-                printError(pkg, 'incoherent-logrotate-file', f)
-            link=enreg[3]
+            res = logrotate_regex.search(f)
+            if res:
+                logrotate_file += 1
+                if res.group(1) != pkg.name:
+                    printError(pkg, 'incoherent-logrotate-file', f)
+
             if link != '':
-                ext=compr_regex.search(link)
+                ext = compr_regex.search(link)
                 if ext:
                     if not re.compile('\.' + ext.group(1) + '$').search(f):
                         printError(pkg, 'compressed-symlink-with-wrong-ext', f, link)
 
-            perm=mode & 07777
+            perm = mode & 07777
 
             # bit s check
             if stat.S_ISGID & mode or stat.S_ISUID & mode:
                 # check only normal files
                 if stat.S_ISREG(mode):
-                    user=enreg[1]
-                    group=enreg[2]
-                    setuid=None
-                    setgid=None
+                    setuid = None
+                    setgid = None
                     if stat.S_ISUID & mode:
-                        setuid=user
+                        setuid = user
                     if stat.S_ISGID & mode:
-                        setgid=group
+                        setgid = group
                     if setuid:
                         printError(pkg, 'setuid-binary', f, setuid, oct(perm))
                     if setgid:
@@ -389,16 +396,33 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                         printError(pkg, 'non-standard-executable-perm', f, oct(perm))
 
             if log_regex.search(f):
-                log_file=f
+                log_file = f
+
+            # Hardlink check
+            hardlink = hardlinks.get((rdev, inode))
+            if hardlink and os.path.dirname(hardlink) != os.path.dirname(f):
+                printWarning(pkg, 'cross-directory-hard-link', f, hardlink)
+            hardlinks[(rdev, inode)] = f
 
             # normal file check
             if stat.S_ISREG(mode):
 
+                # Prefetch scriptlets, strip quotes from them (#169)
+                postin = pkg[rpm.RPMTAG_POSTIN] or pkg[rpm.RPMTAG_POSTINPROG]
+                if postin:
+                    postin = quotes_regex.sub('', postin)
+                postun = pkg[rpm.RPMTAG_POSTUN] or pkg[rpm.RPMTAG_POSTUNPROG]
+                if postun:
+                    postun = quotes_regex.sub('', postun)
+                preun = pkg[rpm.RPMTAG_PREUN] or pkg[rpm.RPMTAG_PREUNPROG]
+                if preun:
+                    preun = quotes_regex.sub('', preun)
+
                 if not devel_pkg:
                     if lib_path_regex.search(f):
-                        lib_file=1
+                        lib_file = 1
                     elif not is_doc:
-                        non_lib_file=f
+                        non_lib_file = f
 
                 if log_regex.search(f):
                     nonexec_file = 1
@@ -416,14 +440,12 @@ class FilesCheck(AbstractCheck.AbstractCheck):
 
                 # check ldconfig call in %post and %postun
                 if lib_regex.search(f):
-                    postin=pkg[rpm.RPMTAG_POSTIN] or pkg[rpm.RPMTAG_POSTINPROG]
                     if not postin:
                         printError(pkg, 'library-without-ldconfig-postin', f)
                     else:
                         if not ldconfig_regex.search(postin):
                             printError(pkg, 'postin-without-ldconfig', f)
 
-                    postun=pkg[rpm.RPMTAG_POSTUN] or pkg[rpm.RPMTAG_POSTUNPROG]
                     if not postun:
                         printError(pkg, 'library-without-ldconfig-postun', f)
                     else:
@@ -431,37 +453,35 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                             printError(pkg, 'postun-without-ldconfig', f)
 
                 # check depmod call in %post and %postun
-                res=not is_kernel_package and kernel_modules_regex.search(f)
+                res = not is_kernel_package and kernel_modules_regex.search(f)
                 if res:
-                    kernel_version=res.group(1)
-                    kernel_version_regex=re.compile('depmod -a.*F /boot/System.map-' + kernel_version + '.*' + kernel_version, re.MULTILINE | re.DOTALL)
-                    postin=pkg[rpm.RPMTAG_POSTIN] or pkg[rpm.RPMTAG_POSTINPROG]
+                    kernel_version = res.group(1)
+                    kernel_version_regex = re.compile(
+                        '\\bdepmod\s+-a.*F\s+/boot/System\.map-' +
+                        re.escape(kernel_version) + '\\b.*\\b' +
+                        re.escape(kernel_version) + '\\b',
+                        re.MULTILINE | re.DOTALL)
+
                     if not postin or not depmod_regex.search(postin):
                         printError(pkg, 'module-without-depmod-postin', f)
                     # check that we run depmod on the right kernel
-                    else:
-                        if not kernel_version_regex.search(postin):
-                            printError(pkg, 'postin-with-wrong-depmod', f)
+                    elif not kernel_version_regex.search(postin):
+                        printError(pkg, 'postin-with-wrong-depmod', f)
 
-                    postun=pkg[rpm.RPMTAG_POSTUN] or pkg[rpm.RPMTAG_POSTUNPROG]
                     if not postun or not depmod_regex.search(postun):
                         printError(pkg, 'module-without-depmod-postun', f)
                     # check that we run depmod on the right kernel
-                    else:
-                        if not kernel_version_regex.search(postun):
-                            printError(pkg, 'postun-with-wrong-depmod', f)
+                    elif not kernel_version_regex.search(postun):
+                        printError(pkg, 'postun-with-wrong-depmod', f)
 
                 # check install-info call in %post and %postun
                 if f.startswith('/usr/share/info/'):
-                    postin=pkg[rpm.RPMTAG_POSTIN]
                     if not postin:
                         printError(pkg, 'info-files-without-install-info-postin', f)
                     else:
                         if not install_info_regex.search(postin):
                             printError(pkg, 'postin-without-install-info', f)
 
-                    postun=pkg[rpm.RPMTAG_POSTUN]
-                    preun=pkg[rpm.RPMTAG_PREUN]
                     if not postun and not preun:
                         printError(pkg, 'info-files-without-install-info-postun', f)
                     else:
@@ -469,17 +489,23 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                            (not preun or not install_info_regex.search(preun)):
                             printError(pkg, 'postin-without-install-info', f)
 
-
                 # check perl temp file
-                if perl_temp_file.search(f):
+                if perl_temp_file_regex.search(f):
                     printWarning(pkg, 'perl-temp-file', f)
 
+                is_buildconfig = buildconfigfile_regex.search(f) and True
+
+                # check rpaths in buildconfig files
+                if is_buildconfig:
+                    ln = pkg.grep(buildconfig_rpath_regex, f)
+                    if ln:
+                        printError(pkg, 'rpath-in-buildconfig', f, 'lines', ln)
+
                 if bin_regex.search(f) and mode & 0111 == 0:
                     printWarning(pkg, 'non-executable-in-bin', f, oct(perm))
                 if not devel_pkg and not is_doc and \
                        (includefile_regex.search(f) or \
-                        develfile_regex.search(f) or \
-                        buildconfigfile_regex.search(f)):
+                        develfile_regex.search(f) or is_buildconfig):
                     printWarning(pkg, 'devel-file-in-non-devel-package', f)
                 if mode & 0444 != 0444 and perm & 07000 == 0 and f[0:len('/var/log')] != '/var/log':
                     printError(pkg, 'non-readable', f, oct(perm))
@@ -490,7 +516,7 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                     printError(pkg, 'world-writable', f, oct(perm))
 
                 if not perl_dep_error:
-                    res=perl_regex.search(f)
+                    res = perl_regex.search(f)
                     if res:
                         if perl_version_trick:
                             vers = res.group(1) + '.' + res.group(2)
@@ -499,15 +525,15 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                         if not (pkg.check_versioned_dep('perl-base', vers) or
                                 pkg.check_versioned_dep('perl', vers)):
                             printError(pkg, 'no-dependency-on', 'perl-base', vers)
-                            perl_dep_error=1
+                            perl_dep_error = 1
 
                 if not python_dep_error:
-                    res=python_regex.search(f)
+                    res = python_regex.search(f)
                     if res:
                         if not (pkg.check_versioned_dep('python-base', res.group(1)) or
                                 pkg.check_versioned_dep('python', res.group(1))):
                             printError(pkg, 'no-dependency-on', 'python-base', res.group(1))
-                            python_dep_error=1
+                            python_dep_error = 1
 
                 # normal executable check
                 if mode & stat.S_IXUSR and perm != 0755:
@@ -549,65 +575,65 @@ class FilesCheck(AbstractCheck.AbstractCheck):
 
             # symbolic link check
             elif stat.S_ISLNK(mode):
-                r=absolute_regex.search(link)
-                is_so=sofile_regex.search(f)
+                r = absolute_regex.search(link)
+                is_so = sofile_regex.search(f)
                 if not devel_pkg and is_so and not link.endswith('.so'):
                     printWarning(pkg, 'devel-file-in-non-devel-package', f)
                 # absolute link
                 if r:
-                    if (not is_so) and link not in files.keys() and link not in req_names:
-                        is_exception=0
+                    if (not is_so) and link not in files and link not in req_names:
+                        is_exception = 0
                         for e in dangling_exceptions:
                             if e[0].search(link):
-                                is_exception=e[1]
+                                is_exception = e[1]
                                 break
                         if is_exception:
                             if is_exception not in req_names:
                                 printWarning(pkg, 'no-dependency-on', is_exception)
                         else:
                             printWarning(pkg, 'dangling-symlink', f, link)
-                    linktop=r.group(1)
-                    r=absolute_regex.search(f)
+                    linktop = r.group(1)
+                    r = absolute_regex.search(f)
                     if r:
-                        filetop=r.group(1)
+                        filetop = r.group(1)
                         if filetop == linktop or use_relative_symlinks:
-                            printWarning(pkg ,'symlink-should-be-relative', f, link)
+                            printWarning(pkg, 'symlink-should-be-relative', f, link)
                 # relative link
                 else:
                     if not is_so:
-                        extractedfile = '%s%s/%s' % (pkg.dirName(), os.path.dirname(f), link)
+                        extractedfile = os.path.join(os.path.dirname(pkgfile.path), link)
                         extractedfile = os.path.normpath(extractedfile)
                         pkgfile = '%s/%s' % (os.path.dirname(f), link)
                         pkgfile = os.path.normpath(pkgfile)
-                        if not (files.has_key(pkgfile) or os.path.exists(extractedfile) or pkgfile in req_names):
-                            is_exception=0
+                        if not (pkgfile in files or os.path.exists(extractedfile) or pkgfile in req_names):
+                            is_exception = 0
                             for e in dangling_exceptions:
                                 if e[0].search(link):
-                                    is_exception=e[1]
+                                    is_exception = e[1]
                                     break
                             if is_exception:
                                 if not is_exception in req_names:
                                     printWarning(pkg, 'no-dependency-on', is_exception)
                             else:
                                 printWarning(pkg, 'dangling-relative-symlink', f, link)
-                    pathcomponents=string.split(f, '/')[1:]
-                    r=points_regex.search(link)
-                    lastpop=None
-                    mylink=None
+                    pathcomponents = f.split('/')[1:]
+                    r = points_regex.search(link)
+                    lastpop = None
+                    mylink = None
 
                     while r:
-                        mylink=r.group(1)
+                        mylink = r.group(1)
                         if len(pathcomponents) == 0:
                             printError(pkg, 'symlink-has-too-many-up-segments', f, link)
                             break
                         else:
-                            lastpop=pathcomponents[0]
-                            pathcomponents=pathcomponents[1:]
-                            r=points_regex.search(mylink)
+                            lastpop = pathcomponents[0]
+                            pathcomponents = pathcomponents[1:]
+                            r = points_regex.search(mylink)
 
                     if mylink and lastpop:
-                        r=absolute2_regex.search(mylink)
-                        linktop=r.group(1)
+                        r = absolute2_regex.search(mylink)
+                        linktop = r.group(1)
 
                         # does the link go up and then down into the same directory?
                         #if linktop == lastpop:
@@ -619,13 +645,13 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                                 # relative link into other toplevel directory
                                 printWarning(pkg, 'symlink-should-be-absolute', f, link)
                         # check additional segments for mistakes like `foo/../bar/'
-                        for linksegment in string.split(mylink, '/'):
+                        for linksegment in mylink.split('/'):
                             if linksegment == '..':
                                 printError(pkg, 'symlink-contains-up-and-down-segments', f, link)
 
             # check text file
             if stat.S_ISREG(mode):
-                path=pkg.dirName() + '/' + f
+                path = pkgfile.path
                 if os.access(path, os.R_OK):
                     if istextfile(path, pkg):
                         fobj = open(path, 'r')
@@ -633,10 +659,10 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                             line = fobj.readline()
                         finally:
                             fobj.close()
-                        res=None
+                        res = None
                         # ignore perl module shebang -- TODO: disputed...
                         if not f.endswith('.pm'):
-                            res=shebang_regex.search(line)
+                            res = shebang_regex.search(line)
                         # sourced scripts should not be executable
                         if sourced_script_regex.search(f):
                             if res:
@@ -653,10 +679,10 @@ class FilesCheck(AbstractCheck.AbstractCheck):
 
                             if mode & 0111 == 0 and not is_doc:
                                 printError(pkg, 'non-executable-script', f, oct(perm))
-                            if line.endswith('\r\n'):
+                            if line.endswith('\r\n') or line.endswith('\r'):
                                 printError(pkg, 'wrong-script-end-of-line-encoding', f)
-                        elif is_doc and not re.search('\.(?:rtf|x?html?)$', f, re.I):
-                            if line.endswith('\r\n'):
+                        elif is_doc and not skipdocs_regex.search(f):
+                            if line.endswith('\r\n') or line.endswith('\r'):
                                 printWarning(pkg, 'wrong-file-end-of-line-encoding', f)
                             # We check only doc text files for UTF-8-ness;
                             # checking everything may be slow and can generate
@@ -665,9 +691,11 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                                 printWarning(pkg, 'file-not-utf8', f)
 
                     elif is_doc and compr_regex.search(f):
-                        # compressed docs, eg. info and man files etc
-                        if use_utf8 and not is_utf8(path):
-                            printWarning(pkg, 'file-not-utf8', f)
+                        ff = compr_regex.sub('', f)
+                        if not skipdocs_regex.search(ff):
+                            # compressed docs, eg. info and man files etc
+                            if use_utf8 and not is_utf8(path):
+                                printWarning(pkg, 'file-not-utf8', f)
 
 
             if f.startswith('/etc/cron.d/'):
@@ -687,10 +715,9 @@ class FilesCheck(AbstractCheck.AbstractCheck):
             printError(pkg, 'outside-libdir-files', non_lib_file)
 
 # Create an object to enable the auto registration of the test
-check=FilesCheck()
+check = FilesCheck()
 
-if Config.info:
-    addDetails(
+addDetails(
 'no-documentation',
 '''The package contains no documentation (README, doc, etc).
 You have to include documentation files.''',
@@ -702,12 +729,12 @@ the standard %doc tag.''',
 'non-standard-uid',
 '''A file in this package is owned by a non standard user.
 Standard users are:
-%s''' % fill(", ".join(standard_users)),
+%s.''' % ", ".join(standard_users),
 
 'non-standard-gid',
 '''A file in this package is owned by a non standard group.
 Standard groups are:
-%s''' % fill(", ".join(standard_groups)),
+%s.''' % ", ".join(standard_groups),
 
 'library-without-ldconfig-postin',
 '''This package contains a library and provides no %post scriptlet containing
@@ -780,7 +807,7 @@ in the package. Move these files out of the package and rebuild it.''',
 
 'htaccess-file',
 '''You have individual apache configuration .htaccess file(s) in your package.
-Replace them by a central configuration file in /etc/, according to the web 
+Replace them by a central configuration file in /etc/, according to the web
 application packaging policy for your distribution.''',
 
 'info-dir-file',
@@ -797,11 +824,11 @@ files. Mark the file as %config in the spec file.''',
 '''The symlink points to a compressed file but doesn't use the same extension.''',
 
 'setuid-binary',
-'''The file is setuid, this may be dangerous, especially if this 
+'''The file is setuid, this may be dangerous, especially if this
 file is setuid root.''',
 
 'setgid-binary',
-'''The file is setgid. Usually this is a packaging bug. If this is a game, 
+'''The file is setgid. Usually this is a packaging bug. If this is a game,
 then, you should use the proper rpm group, or location.''',
 
 'non-standard-executable-perm',
@@ -841,11 +868,17 @@ something non-standard.''',
 '''
 ''',
 
+'cross-directory-hard-link',
+'''File is hard linked across directories.  This can cause problems in
+installations where the directories are located on different devices.''',
+
 'dangling-symlink',
 '''The symbolic link points nowhere.''',
 
 'symlink-should-be-relative',
-'''Absolute symlinks are problematic eg. when working with chroot environments.''',
+'''Absolute symlinks are problematic eg. when working with chroot environments.
+symlinks(8) is a tool that can be useful for creating/dealing with relative
+symlinks at package build time.''',
 
 'dangling-relative-symlink',
 '''The relative symbolic link points nowhere.''',
@@ -961,7 +994,7 @@ correctly in some circumstances.''',
 
 'file-not-utf8',
 '''The character encoding of this file is not UTF-8.  Consider converting it
-in the specfile for example using iconv(1).''',
+in the specfile's %prep section for example using iconv(1).''',
 
 'filename-not-utf8',
 '''The character encoding of the name of this file is not UTF-8.  Rename it.''',
@@ -986,16 +1019,20 @@ but it actually is not in the filesystem.  Because of this, some checks will
 be skipped.''',
 
 'executable-crontab-file',
-'''This crontab file has executable bit set, which is refused by newer version 
+'''This crontab file has executable bit set, which is refused by newer version
 of cron''',
 
 'non-owner-writeable-only-crontab-file',
-'''This crontab file is writeable by other users as its owner, which is refused 
+'''This crontab file is writeable by other users as its owner, which is refused
 by newer version of cron and insecure''',
 
 'symlink-crontab-file',
-'''This crontab file is a symbolic link, which is insecure and refused by newer 
+'''This crontab file is a symbolic link, which is insecure and refused by newer
 version of cron''',
+
+'rpath-in-buildconfig',
+'''This build configuration file contains rpaths which will be introduced into 
+dependent packages.''',
 )
 
 # FilesCheck.py ends here
