@@ -1,21 +1,25 @@
+# -*- coding: utf-8 -*-
 #---------------------------------------------------------------
 # Project         : Mandriva Linux
 # Module          : rpmlint
 # File            : MenuCheck.py
-# Version         : $Id: MenuCheck.py 1313 2007-02-16 19:19:42Z scop $
+# Version         : $Id: MenuCheck.py 1544 2009-02-01 22:05:18Z scop $
 # Author          : Frederic Lepied
 # Created On      : Mon Mar 20 07:43:37 2000
 #---------------------------------------------------------------
 
-from Filter import *
-import AbstractCheck
-import Pkg
-import rpm
 import re
-import string
 import stat
 
-DEFAULT_VALID_SECTIONS=(
+import rpm
+
+from Filter import addDetails, printError, printInfo, printWarning
+import AbstractCheck
+import Config
+import Pkg
+
+
+DEFAULT_VALID_SECTIONS = (
     'Office/Accessories',
     'Office/Address Books',
     'Office/Communications/Fax',
@@ -127,32 +131,30 @@ DEFAULT_LAUNCHERS = (['(?:/usr/bin/)?kdesu', ('/usr/bin/kdesu', 'kdesu')],
                      ['(?:/usr/bin/)?soundwrapper', None],
                     )
 
-menu_file_regex=re.compile('^/usr/lib/menu/([^/]+)$')
-old_menu_file_regex=re.compile('^/usr/share/(gnome/apps|applnk)/([^/]+)$')
-package_regex=re.compile('\?package\((.*)\):')
-needs_regex=re.compile('needs=(\"([^\"]+)\"|([^ \t\"]+))')
-section_regex=re.compile('section=(\"([^\"]+)\"|([^ \t\"]+))')
-title_regex=re.compile('[\"\s]title=(\"([^\"]+)\"|([^ \t\"]+))')
-longtitle_regex=re.compile('longtitle=(\"([^\"]+)\"|([^ \t\"]+))')
-command_regex=re.compile('command=(?:\"([^\"]+)\"|([^ \t\"]+))')
-kdesu_command_regex=re.compile('[/usr/bin/]?kdesu -c \"?([^ \t\"]+)\"?')
-kdesu_bin_regex=re.compile('[/usr/bin/]?kdesu')
-icon_regex=re.compile('icon=\"?([^\" ]+)')
-valid_sections=Config.getOption('ValidMenuSections', DEFAULT_VALID_SECTIONS)
-update_menus_regex=re.compile('^[^#]*update-menus',re.MULTILINE)
-standard_needs=Config.getOption('ExtraMenuNeeds', DEFAULT_EXTRA_MENU_NEEDS)
-icon_paths=Config.getOption('IconPath', DEFAULT_ICON_PATH)
-xpm_ext_regex=re.compile('/usr/share/icons/(mini/|large/).*\.xpm$')
-icon_ext_regex=re.compile(Config.getOption('IconFilename', '.*\.png$'))
-version_regex=re.compile('([0-9.][0-9.]+)($|\s)')
-launchers=Config.getOption('MenuLaunchers', DEFAULT_LAUNCHERS)
-bad_title_regex=re.compile('/')
-menu64_file_regex=re.compile('^/usr/lib64/menu')
-xdg_migrated_regex=re.compile('xdg=\"?([^\" ]+)')
+menu_file_regex = re.compile('^/usr/lib/menu/([^/]+)$')
+old_menu_file_regex = re.compile('^/usr/share/(gnome/apps|applnk)/([^/]+)$')
+package_regex = re.compile('\?package\((.*)\):')
+needs_regex = re.compile('needs=(\"([^\"]+)\"|([^ \t\"]+))')
+section_regex = re.compile('section=(\"([^\"]+)\"|([^ \t\"]+))')
+title_regex = re.compile('[\"\s]title=(\"([^\"]+)\"|([^ \t\"]+))')
+longtitle_regex = re.compile('longtitle=(\"([^\"]+)\"|([^ \t\"]+))')
+command_regex = re.compile('command=(?:\"([^\"]+)\"|([^ \t\"]+))')
+icon_regex = re.compile('icon=\"?([^\" ]+)')
+valid_sections = Config.getOption('ValidMenuSections', DEFAULT_VALID_SECTIONS)
+update_menus_regex = re.compile('^[^#]*update-menus', re.MULTILINE)
+standard_needs = Config.getOption('ExtraMenuNeeds', DEFAULT_EXTRA_MENU_NEEDS)
+icon_paths = Config.getOption('IconPath', DEFAULT_ICON_PATH)
+xpm_ext_regex = re.compile('/usr/share/icons/(mini/|large/).*\.xpm$')
+icon_ext_regex = re.compile(Config.getOption('IconFilename', '.*\.png$'))
+version_regex = re.compile('([0-9.][0-9.]+)($|\s)')
+launchers = Config.getOption('MenuLaunchers', DEFAULT_LAUNCHERS)
+bad_title_regex = re.compile('/')
+menu64_file_regex = re.compile('^/usr/lib64/menu')
+xdg_migrated_regex = re.compile('xdg=\"?([^\" ]+)')
 
 # compile regexps
 for l in launchers:
-    l[0]=re.compile(l[0])
+    l[0] = re.compile(l[0])
 del l
 
 class MenuCheck(AbstractCheck.AbstractCheck):
@@ -165,143 +167,141 @@ class MenuCheck(AbstractCheck.AbstractCheck):
         if pkg.isSource():
             return
 
-        files=pkg.files()
-        menus=[]
+        files = pkg.files()
+        menus = []
 
-        for f in files.keys():
+        for fname, pkgfile in files.items():
             # Check menu files
-            res=menu_file_regex.search(f)
+            res = menu_file_regex.search(fname)
+            mode = pkgfile.mode
             if res:
-                basename=res.group(1)
-                mode=files[f][0]
+                basename = res.group(1)
                 if not stat.S_ISREG(mode):
-                    printError(pkg, 'non-file-in-menu-dir', f)
+                    printError(pkg, 'non-file-in-menu-dir', fname)
                 else:
                     if basename != pkg.name:
-                        printWarning(pkg, 'non-coherent-menu-filename', f)
+                        printWarning(pkg, 'non-coherent-menu-filename', fname)
                     if mode & 0444 != 0444:
-                        printError(pkg, 'non-readable-menu-file', f)
+                        printError(pkg, 'non-readable-menu-file', fname)
                     if mode & 0111 != 0:
-                        printError(pkg, 'executable-menu-file', f)
-                    menus.append(f)
+                        printError(pkg, 'executable-menu-file', fname)
+                    menus.append(fname)
             else:
                 # Check old menus from KDE and GNOME
-                res=old_menu_file_regex.search(f)
+                res = old_menu_file_regex.search(fname)
                 if res:
-                    mode=files[f][0]
                     if stat.S_ISREG(mode):
-                        printError(pkg, 'old-menu-entry', f)
+                        printError(pkg, 'old-menu-entry', fname)
                 else:
                     # Check non transparent xpm files
-                    res=xpm_ext_regex.search(f)
+                    res = xpm_ext_regex.search(fname)
                     if res:
-                        mode=files[f][0]
-                        if stat.S_ISREG(mode) and not pkg.grep('None",', f):
-                            printWarning(pkg, 'non-transparent-xpm', f)
-                if menu64_file_regex.search(f):
-                    printError(pkg, 'menu-in-wrong-dir', f)
-        if len(menus) > 0:
+                        if stat.S_ISREG(mode) and not pkg.grep('None",', fname):
+                            printWarning(pkg, 'non-transparent-xpm', fname)
+                if menu64_file_regex.search(fname):
+                    printError(pkg, 'menu-in-wrong-dir', fname)
+
+        if menus:
+            postin = pkg[rpm.RPMTAG_POSTIN] or pkg[rpm.RPMTAG_POSTINPROG]
+            if not postin:
+                printError(pkg, 'menu-without-postin')
+            else:
+                if not update_menus_regex.search(postin):
+                    printError(pkg, 'postin-without-update-menus')
+
+            postun = pkg[rpm.RPMTAG_POSTUN] or pkg[rpm.RPMTAG_POSTUNPROG]
+            if not postun:
+                printError(pkg, 'menu-without-postun')
+            else:
+                if not update_menus_regex.search(postun):
+                    printError(pkg, 'postun-without-update-menus')
+
             directory = pkg.dirName()
-            if menus != []:
-                postin=pkg[rpm.RPMTAG_POSTIN] or pkg[rpm.RPMTAG_POSTINPROG]
-                if not postin:
-                    printError(pkg, 'menu-without-postin')
-                else:
-                    if not update_menus_regex.search(postin):
-                        printError(pkg, 'postin-without-update-menus')
-
-                postun=pkg[rpm.RPMTAG_POSTUN] or pkg[rpm.RPMTAG_POSTUNPROG]
-                if not postun:
-                    printError(pkg, 'menu-without-postun')
-                else:
-                    if not update_menus_regex.search(postun):
-                        printError(pkg, 'postun-without-update-menus')
-
             for f in menus:
                 # remove comments and handle cpp continuation lines
-                sts, cmd = Pkg.getstatusoutput(('/lib/cpp', directory + f), 1)
-                                                  
-                for line in string.split(cmd, '\n'):
+                cmd = Pkg.getstatusoutput(('/lib/cpp', directory + f), 1)[1]
+
+                for line in cmd.splitlines():
                     if not line.startswith('?'): continue
-                    res=package_regex.search(line)
+                    res = package_regex.search(line)
                     if res:
-                        package=res.group(1)
+                        package = res.group(1)
                         if package != pkg.name:
                             printWarning(pkg, 'incoherent-package-value-in-menu', package, f)
                     else:
                         printInfo(pkg, 'unable-to-parse-menu-entry', line)
 
-                    command=1
-                    res=command_regex.search(line)
+                    command = 1
+                    res = command_regex.search(line)
                     if res:
-                        command_line=string.split(res.group(1) or res.group(2))
-                        command=command_line[0]
+                        command_line = (res.group(1) or res.group(2)).split()
+                        command = command_line[0]
                         for launcher in launchers:
                             if launcher[0].search(command):
-                                found=0
+                                found = 0
                                 if launcher[1]:
-                                    if (files.has_key('/bin/' + command_line[0]) or
-                                        files.has_key('/usr/bin/' + command_line[0]) or
-                                        files.has_key('/usr/X11R6/bin/' + command_line[0])):
-                                        found=1
+                                    if ('/bin/' + command_line[0] in files or
+                                        '/usr/bin/' + command_line[0] in files or
+                                        '/usr/X11R6/bin/' + command_line[0] in files):
+                                        found = 1
                                     else:
                                         for l in launcher[1]:
                                             if l in pkg.req_names():
-                                                found=1
+                                                found = 1
                                                 break
                                     if not found:
                                         printError(pkg, 'use-of-launcher-in-menu-but-no-requires-on', launcher[1][0])
-                                command=command_line[1]
+                                command = command_line[1]
                                 break
                         if command[0] == '/':
-                            if not command in files.keys():
+                            if not command in files:
                                 printWarning(pkg, 'menu-command-not-in-package', command)
                         else:
-                            if not (files.has_key('/bin/' + command) or
-                                    files.has_key('/usr/bin/' + command) or
-                                    files.has_key('/usr/X11R6/bin/' + command)):
+                            if not ('/bin/' + command in files or
+                                    '/usr/bin/' + command in files or
+                                    '/usr/X11R6/bin/' + command in files):
                                 printWarning(pkg, 'menu-command-not-in-package', command)
                     else:
                         printWarning(pkg, 'missing-menu-command')
-                        command=0
+                        command = 0
 
-                    res=longtitle_regex.search(line)
+                    res = longtitle_regex.search(line)
                     if res:
-                        grp=res.groups()
-                        title=grp[1] or grp[2]
+                        grp = res.groups()
+                        title = grp[1] or grp[2]
                         if title[0] != title[0].upper():
                             printWarning(pkg, 'menu-longtitle-not-capitalized', title)
-                        res=version_regex.search(title)
+                        res = version_regex.search(title)
                         if res:
                             printWarning(pkg, 'version-in-menu-longtitle', title)
                     else:
                         printError(pkg, 'no-longtitle-in-menu', f)
-                        title=None
+                        title = None
 
-                    res=title_regex.search(line)
+                    res = title_regex.search(line)
                     if res:
-                        grp=res.groups()
-                        title=grp[1] or grp[2]
+                        grp = res.groups()
+                        title = grp[1] or grp[2]
                         if title[0] != title[0].upper():
                             printWarning(pkg, 'menu-title-not-capitalized', title)
-                        res=version_regex.search(title)
+                        res = version_regex.search(title)
                         if res:
                             printWarning(pkg, 'version-in-menu-title', title)
                         if bad_title_regex.search(title):
                             printError(pkg, 'invalid-title', title)
                     else:
                         printError(pkg, 'no-title-in-menu', f)
-                        title=None
+                        title = None
 
-                    res=needs_regex.search(line)
+                    res = needs_regex.search(line)
                     if res:
-                        grp=res.groups()
-                        needs=string.lower(grp[1] or grp[2])
+                        grp = res.groups()
+                        needs = (grp[1] or grp[2]).lower()
                         if needs in ('x11', 'text' ,'wm'):
-                            res=section_regex.search(line)
+                            res = section_regex.search(line)
                             if res:
-                                grp=res.groups()
-                                section=grp[1] or grp[2]
+                                grp = res.groups()
+                                section = grp[1] or grp[2]
                                 # don't warn entries for sections
                                 if command:
                                     if section not in valid_sections:
@@ -313,21 +313,21 @@ class MenuCheck(AbstractCheck.AbstractCheck):
                     else:
                         printInfo(pkg, 'unable-to-parse-menu-needs', line)
 
-                    res=icon_regex.search(line)
+                    res = icon_regex.search(line)
                     if res:
-                        icon=res.group(1)
+                        icon = res.group(1)
                         if not icon_ext_regex.search(icon):
                             printWarning(pkg, 'invalid-menu-icon-type', icon)
                         if icon[0] == '/' and needs == 'x11':
                             printWarning(pkg, 'hardcoded-path-in-menu-icon', icon)
                         else:
                             for path in icon_paths:
-                                if not (path[0] + icon) in files.keys():
+                                if not (path[0] + icon) in files:
                                     printError(pkg, path[1] + '-icon-not-in-package', icon, f)
                     else:
                         printWarning(pkg, 'no-icon-in-menu', title)
-                    
-                    res=xdg_migrated_regex.search(line)
+
+                    res = xdg_migrated_regex.search(line)
                     if res:
                         if not res.group(1).lower() == "true":
                             printError(pkg, 'non-xdg-migrated-menu')
@@ -336,10 +336,9 @@ class MenuCheck(AbstractCheck.AbstractCheck):
 
 
 # Create an object to enable the auto registration of the test
-check=MenuCheck()
+check = MenuCheck()
 
-if Config.info:
-    addDetails(
+addDetails(
 'non-file-in-menu-dir',
 '''/usr/lib/menu must not contain anything else than normal files.''',
 
