@@ -1,15 +1,16 @@
+# -*- coding: utf-8 -*-
 # Version         : $Id$
 
 #
 # check xdg file format violation
-# 
+#
 # http://standards.freedesktop.org/desktop-entry-spec/desktop-entry-spec-latest.html
 #
 
-from Filter import *
+from Filter import addDetails, printError
 from Pkg import getstatusoutput, is_utf8
 import AbstractCheck
-import Config
+
 
 class MenuXDGCheck(AbstractCheck.AbstractFilesCheck):
     def __init__(self):
@@ -24,16 +25,17 @@ class MenuXDGCheck(AbstractCheck.AbstractFilesCheck):
                 printError(pkg, 'invalid-desktopfile', filename)
         if not is_utf8(f):
                 printError(pkg, 'non-utf8-desktopfile', filename)
-                
 
-check=MenuXDGCheck()
 
-if Config.info:
-    addDetails(
-        'invalid-desktopfile',
-        '''.desktop file is not valid, check with desktop-file-validate''',
-        'non-utf8-desktopfile',
-        '''.desktop file is not encoded in UTF-8''',)
+check = MenuXDGCheck()
+
+addDetails(
+'invalid-desktopfile',
+'''.desktop file is not valid, check with desktop-file-validate''',
+
+'non-utf8-desktopfile',
+'''.desktop file is not encoded in UTF-8''',
+)
 
 # Local variables:
 # indent-tabs-mode: nil
