@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #---------------------------------------------------------------
 # Project         : Mandriva Linux
 # Module          : rpmlint
@@ -8,29 +9,30 @@
 # Purpose         : Apply pam policy
 #---------------------------------------------------------------
 
-from Filter import *
-import AbstractCheck
 import re
-import Config
+
+from Filter import addDetails, printError
+import AbstractCheck
+
 
 pam_stack_re = re.compile('^\s*[^#].*pam_stack\.so\s*service')
 
 class PamCheck(AbstractCheck.AbstractFilesCheck):
     def __init__(self):
         AbstractCheck.AbstractFilesCheck.__init__(self, "PamCheck", "/etc/pam\.d/.*")
-    
+
     def check_file(self, pkg, filename):
         lines = pkg.grep(pam_stack_re, filename)
         if lines:
             printError(pkg, 'use-old-pam-stack', filename,
                        '(line %s)' % ", ".join(lines))
 
-check=PamCheck()
+check = PamCheck()
 
-if Config.info:
-    addDetails(
-        'use-old-pam-stack',
-        '''Update pam file to use include instead of pam_stack.''',)
+addDetails(
+'use-old-pam-stack',
+'''Update pam file to use include instead of pam_stack.''',
+)
 
 # Local variables:
 # indent-tabs-mode: nil
