@@ -1,70 +1,48 @@
+# -*- coding: utf-8 -*-
 #############################################################################
 # File          : Pkg.py
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Tue Sep 28 07:18:06 1999
-# Version       : $Id: Pkg.py 1387 2007-12-06 11:34:48Z scop $
+# Version       : $Id: Pkg.py 1567 2009-03-17 23:20:11Z scop $
 # Purpose       : provide an API to handle a rpm package either by accessing
 #                 the rpm file or by accessing the files contained inside.
 #############################################################################
 
-import os
-import rpm
-import os.path
-import stat
 import commands
-import popen2
+import os
 import re
-import string
+import stat
+import sys
 import tempfile
 import types
-import sys
+import subprocess
+
+try:
+    import magic
+    _magic = magic.open(magic.MAGIC_NONE)
+    _magic.load()
+except:
+    _magic = None
+import rpm
 
 from Filter import printWarning
 
-RPMFILE_CONFIG=(1 << 0)
-RPMFILE_DOC=(1 << 1)
-RPMFILE_DONOTUSE=(1 << 2)
-RPMFILE_MISSINGOK=(1 << 3)
-RPMFILE_NOREPLACE=(1 << 4)
-RPMFILE_SPECFILE=(1 << 5)
-RPMFILE_GHOST=(1 << 6)
-RPMFILE_LICENSE=(1 << 7)
-RPMFILE_README=(1 << 8)
-
-# check if we use a rpm version compatible with 3.0.4
-try:
-    if rpm.RPMTAG_OLDFILENAMES:
-        v304=1
-except AttributeError:
-    v304=0
-
-try:
-    if rpm.RPMSENSE_SCRIPT_PRE:
-        PREREQ_FLAG=rpm.RPMSENSE_PREREQ|rpm.RPMSENSE_SCRIPT_PRE|rpm.RPMSENSE_SCRIPT_POST|rpm.RPMSENSE_SCRIPT_PREUN|rpm.RPMSENSE_SCRIPT_POSTUN
-except AttributeError:
-    PREREQ_FLAG=rpm.RPMSENSE_PREREQ
-
-# check if we use a rpm version compatible with 4.2
-v42 = 0
-try:
-    if rpm.RPMTAG_DISTTAG: # in >= 4.4
-        v42 = 1
-except AttributeError:
-    try:
-        if rpm.RPMTAG_SOURCEPACKAGE: # in 4.2 but not in 4.4.something (6?)
-            v42 = 1
-    except AttributeError:
-        pass
 
 # utilities
 
-var_regex=re.compile('^(.*)\${?(\w+)}?(.*)$')
+PREREQ_FLAG = rpm.RPMSENSE_PREREQ | \
+              rpm.RPMSENSE_SCRIPT_PRE | \
+              rpm.RPMSENSE_SCRIPT_POST | \
+              rpm.RPMSENSE_SCRIPT_PREUN | \
+              rpm.RPMSENSE_SCRIPT_POSTUN
+
+var_regex = re.compile('^(.*)\${?(\w+)}?(.*)$')
 
 def shell_var_value(var, script):
-    assign_regex=re.compile('\\b' + re.escape(var) + '\s*=\s*(.+)\s*(#.*)*$',
-                            re.MULTILINE)
-    res=assign_regex.search(script)
+    assign_regex = re.compile('\\b' + re.escape(var) + '\s*=\s*(.+)\s*(#.*)*$',
+                              re.MULTILINE)
+    res = assign_regex.search(script)
     if res:
         res2 = var_regex.search(res.group(1))
         if res2:
@@ -75,36 +53,38 @@ def shell_var_value(var, script):
         return None
 
 def substitute_shell_vars(val, script):
-    res=var_regex.search(val)
+    res = var_regex.search(val)
     if res:
-        value=shell_var_value(res.group(2), script)
+        value = shell_var_value(res.group(2), script)
         if not value:
-            value=''
+            value = ''
         return res.group(1) + value + substitute_shell_vars(res.group(3), script)
     else:
         return val
 
-def getstatusoutput(cmd, stdoutonly=0):
+def getstatusoutput(cmd, stdoutonly = 0):
     '''A version of commands.getstatusoutput() which can take cmd as a
-       sequence, thus making it potentially more secure.  See popen2.'''
+       sequence, thus making it potentially more secure.'''
     if stdoutonly:
-        proc = popen2.Popen3(cmd)
+        proc = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, close_fds=True)
     else:
-        proc = popen2.Popen4(cmd)
-    proc.tochild.close()
-    text = proc.fromchild.read()
+        proc = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
+    proc.stdin.close()
+    text = proc.stdout.read()
     sts = proc.wait()
     if sts is None: sts = 0
-    if text[-1:] == '\n': text = text[:-1]
+    if text.endswith('\n'):
+        text = text[:-1]
     return sts, text
 
-bz2_regex=re.compile('\.t?bz2?$')
+bz2_regex = re.compile('\.t?bz2?$')
 
 # TODO: is_utf8 could probably be implemented natively without iconv...
 
 def is_utf8(fname):
-    cat='gzip -dcf'
-    if bz2_regex.search(fname): cat='bzip2 -dcf'
+    cat = 'gzip -dcf'
+    if bz2_regex.search(fname): cat = 'bzip2 -dcf'
+    if fname.endswith('lzma'): cat = 'lzma -dc'
     # TODO: better shell escaping or sequence based command invocation
     cmd = commands.getstatusoutput('%s "%s" | iconv -f utf-8 -t utf-8 -o /dev/null' % (cat, fname))
     return not cmd[0]
@@ -150,59 +130,64 @@ def readlines(path):
         fobj.close()
 
 def mktemp():
-    suffix = ".rpmlint%s" % os.getpid()
-    if hasattr(tempfile, "mkstemp"): # Python >= 2.3
-        tmpfd, tmpname = tempfile.mkstemp(suffix)
-        tmpfile = os.fdopen(tmpfd, 'w')
-    else:
-        tmpname = tempfile.mktemp(suffix)
-        tmpfile = open(tmpname, 'w')
+    tmpfd, tmpname = tempfile.mkstemp(prefix = 'rpmlint.')
+    tmpfile = os.fdopen(tmpfd, 'w')
     return tmpfile, tmpname
 
+def get_default_valid_rpmgroups(filename = ""):
+    """ Get the default rpm group from filename, or from the rpm package if no
+    filename is given"""
+    groups = []
+    if not filename:
+        p = InstalledPkg('rpm')
+        groupsfiles = [x for x in p.files() if x.endswith('/GROUPS')]
+        if groupsfiles:
+            filename = groupsfiles[0]
+    if filename and os.path.exists(filename):
+        fobj = open(filename)
+        try:
+            groups = fobj.read().strip().splitlines()
+        finally:
+            fobj.close()
+        if not 'Development/Debug' in groups:
+            groups.append('Development/Debug')
+            groups.sort()
+    return groups
+
 # classes representing package
 
 class Pkg:
-    file_regex=re.compile('(?:\.)?([^:]+):\s+(.*)')
 
-    def __init__(self, filename, dirname, header=None, is_source=0):
-        self.filename=filename
-        self.extracted=0
-        self.dirname=dirname
-        self.file_info=None
-        self.current_linenum=None
-        self._config_files=None
-        self._doc_files=None
-        self._ghost_files=None
-        self._missing_ok_files=None
-        self._files=None
-        self._requires=None
-        self._req_names=-1
+    def __init__(self, filename, dirname, header = None, is_source = 0):
+        self.filename = filename
+        self.extracted = 0
+        self.dirname = dirname
+        self.current_linenum = None
+        self._config_files = None
+        self._doc_files = None
+        self._noreplace_files = None
+        self._ghost_files = None
+        self._missingok_files = None
+        self._files = None
+        self._requires = None
+        self._req_names = -1
 
         if header:
-            self.header=header
-            self.is_source=is_source
+            self.header = header
+            self.is_source = is_source
         else:
             # Create a package object from the file name
-            if v42:
-                ts=rpm.TransactionSet()
-                # Don't check signatures here...
-                ts.setVSFlags(rpm._RPMVSF_NOSIGNATURES)
-                fd = os.open(filename, os.O_RDONLY)
-                try:
-                    self.header = ts.hdrFromFdno(fd)
-                finally:
-                    os.close(fd)
-                self.is_source = not self.header[rpm.RPMTAG_SOURCERPM]
-            else:
-                fd = os.open(filename, os.O_RDONLY)
-                try:
-                    (self.header, self.is_source) = rpm.headerFromPackage(fd)
-                finally:
-                    os.close(fd)
+            ts = rpm.TransactionSet()
+            # Don't check signatures here...
+            ts.setVSFlags(rpm._RPMVSF_NOSIGNATURES)
+            fd = os.open(filename, os.O_RDONLY)
+            try:
+                self.header = ts.hdrFromFdno(fd)
+            finally:
+                os.close(fd)
+            self.is_source = not self.header[rpm.RPMTAG_SOURCERPM]
 
-        self._lang_files=None
-
-        self.name=self.header[rpm.RPMTAG_NAME]
+        self.name = self.header[rpm.RPMTAG_NAME]
         if self.isNoSource():
             self.arch = 'nosrc'
         elif self.isSource():
@@ -236,38 +221,25 @@ class Pkg:
             self._extract()
         return self.dirname
 
-    # handle the extract phasis
+    # extract rpm contents
     def _extract(self):
-        s=os.stat(self.dirname)
+        s = os.stat(self.dirname)
         if not stat.S_ISDIR(s[stat.ST_MODE]):
             sys.stderr.write('unable to access dir %s\n' % self.dirname)
             return None
         else:
-            self.dirname = '%s/%s.%d' % (self.dirname, os.path.basename(self.filename), os.getpid())
-            os.mkdir(self.dirname)
+            self.dirname = tempfile.mkdtemp(
+                prefix = 'rpmlint.%s.' % os.path.basename(self.filename),
+                dir = self.dirname)
             # TODO: better shell escaping or sequence based command invocation
-            command_str='rpm2cpio "%s" | (cd "%s"; cpio -id); chmod -R +rX "%s"' % (self.filename, self.dirname, self.dirname)
-            cmd=commands.getstatusoutput(command_str)
-            self.extracted=1
+            command_str = 'rpm2cpio "%s" | (cd "%s"; cpio -id); chmod -R +rX "%s"' % (self.filename, self.dirname, self.dirname)
+            cmd = commands.getstatusoutput(command_str)
+            self.extracted = 1
             return cmd
 
     def checkSignature(self):
         return getstatusoutput(('env', 'LC_ALL=C', 'rpm', '-K', self.filename))
 
-    # return the array of info returned by the file command on each file
-    def getFilesInfo(self):
-        if self.file_info == None:
-            self.file_info=[]
-            lines = commands.getoutput('cd %s ; find . -type f -print0 | LC_ALL=C xargs -0r file' % self.dirName())
-            lines=string.split(lines, '\n')
-            for l in lines:
-                #print l
-                res=Pkg.file_regex.search(l)
-                if res:
-                    self.file_info.append([res.group(1), res.group(2)])
-            #print self.file_info
-        return self.file_info
-
     # remove the extracted files from the package
     def cleanup(self):
         if self.extracted and self.dirname:
@@ -296,114 +268,104 @@ class Pkg:
     # return the associative array indexed on file names with
     # the values as: (file perm, file owner, file group, file link to)
     def files(self):
-        if self._files != None:
+        if self._files is not None:
             return self._files
+
         self._gatherFilesInfo()
         return self._files
 
     # return the list of config files
     def configFiles(self):
-        if self._config_files != None:
+        if self._config_files is not None:
             return self._config_files
-        self._gatherFilesInfo()
+
+        self._config_files = [x.name for x in self.files().values()
+                              if x.is_config]
         return self._config_files
 
     # return the list of noreplace files
     def noreplaceFiles(self):
-        if self._noreplace_files != None:
+        if self._noreplace_files is not None:
             return self._noreplace_files
-        self._gatherFilesInfo()
+
+        self._noreplace_files = [x.name for x in self.files().values()
+                                 if x.is_noreplace]
         return self._noreplace_files
 
     # return the list of documentation files
     def docFiles(self):
-        if self._doc_files != None:
+        if self._doc_files is not None:
             return self._doc_files
-        self._gatherFilesInfo()
+
+        self._doc_files = [x.name for x in self.files().values() if x.is_doc]
         return self._doc_files
 
     # return the list of ghost files
     def ghostFiles(self):
-        if self._ghost_files != None:
+        if self._ghost_files is not None:
             return self._ghost_files
-        self._gatherFilesInfo()
+
+        self._ghost_files = [x.name for x in self.files().values()
+                             if x.is_ghost]
         return self._ghost_files
 
     def missingOkFiles(self):
-        if self._missing_ok_files != None:
-            return self._missing_ok_files
-        self._gatherFilesInfo()
-        return self._missing_ok_files
+        if self._missingok_files is not None:
+            return self._missingok_files
+
+        self._missingok_files = [x.name for x in self.files().values()
+                                 if x.is_missingok]
+        return self._missingok_files
 
     # extract information about the files
     def _gatherFilesInfo(self):
-        global v304
 
-        self._config_files=[]
-        self._doc_files=[]
-        self._noreplace_files=[]
-        self._ghost_files=[]
-        self._missing_ok_files=[]
-        self._files={}
-        self._files_array=[]
-        flags=self.header[rpm.RPMTAG_FILEFLAGS]
-        modes=self.header[rpm.RPMTAG_FILEMODES]
-        users=self.header[rpm.RPMTAG_FILEUSERNAME]
-        groups=self.header[rpm.RPMTAG_FILEGROUPNAME]
-        links=self.header[rpm.RPMTAG_FILELINKTOS]
-        sizes=self.header[rpm.RPMTAG_FILESIZES]
-        md5s=self.header[rpm.RPMTAG_FILEMD5S]
-        mtimes=self.header[rpm.RPMTAG_FILEMTIMES]
-        rdevs=self.header[rpm.RPMTAG_FILERDEVS]
-        # Get files according to rpm version
-        if v304:
-            files=self.header[rpm.RPMTAG_OLDFILENAMES]
-            if files == None:
-                basenames=self.header[rpm.RPMTAG_BASENAMES]
-                if basenames:
-                    dirnames=self.header[rpm.RPMTAG_DIRNAMES]
-                    dirindexes=self.header[rpm.RPMTAG_DIRINDEXES]
-                    files=[]
-                    # The rpmlib or the python module doesn't report a list for RPMTAG_DIRINDEXES
-                    # if the list has one element...
-                    if type(dirindexes) == types.IntType:
-                        files.append(dirnames[dirindexes] + basenames[0])
-                    else:
-                        for idx in range(0, len(dirindexes)):
-                            files.append(dirnames[dirindexes[idx]] + basenames[idx])
-        else:
-            files=self.header[rpm.RPMTAG_FILENAMES]
+        self._files = {}
+        flags = self.header[rpm.RPMTAG_FILEFLAGS]
+        modes = self.header[rpm.RPMTAG_FILEMODES]
+        users = self.header[rpm.RPMTAG_FILEUSERNAME]
+        groups = self.header[rpm.RPMTAG_FILEGROUPNAME]
+        links = self.header[rpm.RPMTAG_FILELINKTOS]
+        sizes = self.header[rpm.RPMTAG_FILESIZES]
+        md5s = self.header[rpm.RPMTAG_FILEMD5S]
+        mtimes = self.header[rpm.RPMTAG_FILEMTIMES]
+        rdevs = self.header[rpm.RPMTAG_FILERDEVS]
+        langs = self.header[rpm.RPMTAG_FILELANGS]
+        inodes = self.header[rpm.RPMTAG_FILEINODES]
+        deps = self.header[rpm.RPMTAG_FILEREQUIRE]
+        files = self.header[rpm.RPMTAG_FILENAMES]
+        magics = self.header[rpm.RPMTAG_FILECLASS]
+
+        # rpm-python < 4.6 does not return a list for this (or FILEDEVICES,
+        # FWIW) for packages containing exactly one file
+        if not isinstance(inodes, types.ListType):
+            inodes = [inodes]
 
         if files:
-            self._files_array=files
             for idx in range(0, len(files)):
-                if flags[idx] & RPMFILE_CONFIG:
-                    self._config_files.append(files[idx])
-                if flags[idx] & RPMFILE_DOC:
-                    self._doc_files.append(files[idx])
-                if flags[idx] & RPMFILE_NOREPLACE:
-                    self._noreplace_files.append(files[idx])
-                if flags[idx] & RPMFILE_GHOST:
-                    self._ghost_files.append(files[idx])
-                if flags[idx] & RPMFILE_MISSINGOK:
-                    self._missing_ok_files.append(files[idx])
-                self._files[files[idx]]=(modes[idx], users[idx],
-                                         groups[idx], links[idx],
-                                         sizes[idx], md5s[idx],
-                                         mtimes[idx], rdevs[idx])
-
-    def langFiles(self):
-        if self._lang_files == None:
-            self._lang_files={}
-            array=self.header[rpm.RPMTAG_FILELANGS]
-            if array:
-                for idx in range(0, len(array)):
-                    self._lang_files[self._files_array[idx]] = array[idx]
-
-        return self._lang_files
-
-    def fileLang(self, f):
-        return self.langFiles()[f]
+                pkgfile = PkgFile(files[idx])
+                # Do not use os.path.join here, pkgfile.name can start with a
+                # / which would result in self.dirName being ignored
+                pkgfile.path = os.path.normpath(
+                    self.dirName() + '/' + pkgfile.name)
+                pkgfile.flags = flags[idx]
+                pkgfile.mode = modes[idx]
+                pkgfile.user = users[idx]
+                pkgfile.group = groups[idx]
+                pkgfile.linkto = links[idx]
+                pkgfile.size = sizes[idx]
+                pkgfile.md5 = md5s[idx]
+                pkgfile.mtime = mtimes[idx]
+                pkgfile.rdev = rdevs[idx]
+                pkgfile.inode = inodes[idx]
+                pkgfile.deps = deps[idx]
+                pkgfile.lang = langs[idx]
+                pkgfile.magic = magics[idx]
+                if not pkgfile.magic and _magic:
+                    pkgfile.magic = _magic.file(pkgfile.path)
+                if pkgfile.magic is None:
+                    pkgfile.magic = ''
+                self._files[pkgfile.name] = pkgfile
 
     # API to access dependency information
     def obsoletes(self):
@@ -420,15 +382,15 @@ class Pkg:
 
     def req_names(self):
         if self._req_names == -1:
-            self._req_names = map(lambda x: x[0], self.requires() + self.prereq())
+            self._req_names = [x[0] for x in self.requires() + self.prereq()]
         return self._req_names
 
     def check_versioned_dep(self, name, version):
-        for d in self.requires()+self.prereq():
+        for d in self.requires() + self.prereq():
             if d[0] == name:
-                current_version=d[1]
+                current_version = d[1]
                 if current_version.find(':') > 0:
-                    current_version=''.join(current_version.split(':')[1:])
+                    current_version = ''.join(current_version.split(':')[1:])
                 if d[2] & rpm.RPMSENSE_EQUAL != rpm.RPMSENSE_EQUAL or current_version != version:
                     return 0
                 else:
@@ -444,23 +406,20 @@ class Pkg:
         return self._provides
 
     # internal function to gather dependency info used by the above ones
-    def _gather_aux(self, header, list, nametag, versiontag, flagstag, prereq=None):
+    def _gather_aux(self, header, list, nametag, versiontag, flagstag, prereq = None):
         names = header[nametag]
         versions = header[versiontag]
         flags = header[flagstag]
 
         if versions:
-            # workaroung buggy rpm python module that doesn't return a list
-            if type(flags) != types.ListType:
-                flags=[flags]
             for loop in range(len(versions)):
-                if prereq != None and flags[loop] & PREREQ_FLAG:
+                if prereq is not None and flags[loop] & PREREQ_FLAG:
                     prereq.append((names[loop], versions[loop], flags[loop] & (~PREREQ_FLAG)))
                 else:
                     list.append((names[loop], versions[loop], flags[loop]))
 
     def _gatherDepInfo(self):
-        if self._requires == None:
+        if self._requires is None:
             self._requires = []
             self._prereq = []
             self._provides = []
@@ -487,46 +446,36 @@ class Pkg:
 
 def getInstalledPkgs(name):
     """Get list of installed package objects by name."""
+
     pkgs = []
-    if v42:
-        ts = rpm.TransactionSet()
-        tab = ts.dbMatch("name", name)
-        if not tab:
-            raise KeyError, name
-        for hdr in tab:
-            pkgs.append(InstalledPkg(name, hdr))
+    ts = rpm.TransactionSet()
+    if re.search('[?*[]', name):
+        mi = ts.dbMatch()
+        mi.pattern("name", rpm.RPMMIRE_GLOB, name)
     else:
-        db = rpm.opendb()
-        ixs = db.findbyname(name)
-        if not ixs:
-            del db
-            raise KeyError, name
-        for ix in ixs:
-            pkgs.append(InstalledPkg(name, db[ix]))
-        del db
+        mi = ts.dbMatch("name", name)
+    if not mi:
+        raise KeyError(name)
+
+    for hdr in mi:
+        pkgs.append(InstalledPkg(name, hdr))
     return pkgs
 
 # Class to provide an API to an installed package
 class InstalledPkg(Pkg):
-    def __init__(self, name, h=None):
-        if h:
-            Pkg.__init__(self, name, '/', h)
-        else:
-            if v42:
-                ts = rpm.TransactionSet()
-                tab = ts.dbMatch('name', name)
-                if not tab:
-                    raise KeyError, name
-                theHdr = tab.next()
-            else:
-                db = rpm.opendb()
-                tab = db.findbyname(name)
-                if not tab:
-                    del db
-                    raise KeyError, name
-                theHdr = db[tab[0]]
-                del db
-            Pkg.__init__(self, name, '/', theHdr)
+    def __init__(self, name, hdr = None):
+        if not hdr:
+            ts = rpm.TransactionSet()
+            mi = ts.dbMatch('name', name)
+            if not mi:
+                raise KeyError(name)
+            try:
+                hdr = mi.next()
+            except StopIteration:
+                raise KeyError(name)
+
+        Pkg.__init__(self, name, '/', hdr)
+
         self.extracted = 1
         # create a fake filename to satisfy some checks on the filename
         self.filename = '%s-%s-%s.%s.rpm' % (self.name, self[rpm.RPMTAG_VERSION], self[rpm.RPMTAG_RELEASE], self[rpm.RPMTAG_ARCH])
@@ -537,35 +486,46 @@ class InstalledPkg(Pkg):
     def checkSignature(self):
         return (0, 'fake: pgp md5 OK')
 
-    # return the array of info returned by the file command on each file
-    def getFilesInfo(self):
-        if self.file_info == None:
-            self.file_info=[]
-            cmd = ['env', 'LC_ALL=C', 'file']
-            cmd.extend(self.files().keys())
-            sts, lines = getstatusoutput(cmd)
-            #print lines
-            lines=string.split(lines, '\n')
-            for l in lines:
-                #print l
-                res=Pkg.file_regex.search(l)
-                if res:
-                    self.file_info.append([res.group(1), res.group(2)])
-            #print self.file_info
-        return self.file_info
-
 # Class to provide an API to a "fake" package, eg. for specfile-only checks
 class FakePkg:
     def __init__(self, name):
         self.name = name
         self.arch = None
+        self.current_linenum = None
 
     def cleanup(self):
         pass
 
+# Class for files in packages
+class PkgFile(object):
+
+    def __init__(self, name):
+        self.name = name
+        # Real path to the file (taking extract dir into account)
+        self.path = name
+        self.flags = 0
+        self.mode = 0
+        self.user = None
+        self.group = None
+        self.linkto = ''
+        self.size = None
+        self.md5 = None
+        self.mtime = 0
+        self.rdev = ''
+        self.inode = 0
+        self.deps = ''
+        self.lang = ''
+
+    is_config    = property(lambda self: self.flags & rpm.RPMFILE_CONFIG)
+    is_doc       = property(lambda self: self.flags & rpm.RPMFILE_DOC)
+    is_noreplace = property(lambda self: self.flags & rpm.RPMFILE_NOREPLACE)
+    is_ghost     = property(lambda self: self.flags & rpm.RPMFILE_GHOST)
+    is_missingok = property(lambda self: self.flags & rpm.RPMFILE_MISSINGOK)
+
+
 if __name__ == '__main__':
     for p in sys.argv[1:]:
-        pkg=Pkg(sys.argv[1], '/tmp')
+        pkg = Pkg(sys.argv[1], tempfile.gettempdir())
         sys.stdout.write('Requires: %s\n' % pkg.requires())
         sys.stdout.write('Prereq: %s\n' % pkg.prereq())
         sys.stdout.write('Conflicts: %s\n' % pkg.conflicts())
