@@ -1,19 +1,21 @@
+# -*- coding: utf-8 -*-
 #############################################################################
 # File          : SpecCheck.py
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Thu Oct  7 17:06:14 1999
-# Version       : $Id: SpecCheck.py 1355 2007-06-06 22:11:02Z scop $
+# Version       : $Id: SpecCheck.py 1568 2009-03-19 21:05:02Z scop $
 # Purpose       : check the spec file of a source rpm.
 #############################################################################
 
-from Filter import *
-import AbstractCheck
-import Pkg
 import re
-import sys
-import string
+
+from Filter import addDetails, printError, printWarning
+from TagsCheck import VALID_GROUPS
+import AbstractCheck
 import Config
+import Pkg
+
 
 # Don't check for hardcoded library paths in biarch packages
 DEFAULT_BIARCH_PACKAGES = '^(gcc|glibc)'
@@ -22,31 +24,39 @@ DEFAULT_BIARCH_PACKAGES = '^(gcc|glibc)'
 # their noarch files in /usr/lib/<package>/*, or packages that can't
 # be installed on biarch systems
 DEFAULT_HARDCODED_LIB_PATH_EXCEPTIONS = '/lib/(modules|cpp|perl5|rpm|hotplug|firmware)($|[\s/,])'
-
 patch_regex = re.compile("^Patch(\d*)\s*:\s*([^\s]+)", re.IGNORECASE)
-# TODO: http://rpmlint.zarb.org/cgi-bin/trac.cgi/ticket/59
-applied_patch_regex = re.compile("^%patch.*-P\s+(\d+)|^%patch(\d*)\s")
+applied_patch_regex = re.compile("^%patch(\d*)")
+applied_patch_p_regex = re.compile("\s-P\s+(\d+)\\b")
 source_dir_regex = re.compile("^[^#]*(\$RPM_SOURCE_DIR|%{?_sourcedir}?)")
 obsolete_tags_regex = re.compile("^(Copyright|Serial)\s*:\s*([^\s]+)")
 buildroot_regex = re.compile('Buildroot\s*:\s*([^\s]+)', re.IGNORECASE)
 prefix_regex = re.compile('^Prefix\s*:\s*([^\s]+)', re.IGNORECASE)
 packager_regex = re.compile('^Packager\s*:\s*([^\s]+)', re.IGNORECASE)
-make_check_regexp = re.compile('(^|\s|%{?__)make}?\s+(check|test)')
+noarch_regex = re.compile('^BuildArch(?:itectures)?\s*:\s*\\bnoarch\\b', re.IGNORECASE)
+make_check_regex = re.compile('(^|\s|%{?__)make}?\s+(check|test)')
 rm_regex = re.compile('(^|\s)((.*/)?rm|%{?__rm}?) ')
 rpm_buildroot_regex = re.compile('(\\\*)\${?RPM_BUILD_ROOT}?|(%+){?buildroot}?')
 configure_start_regex = re.compile('\./configure')
 configure_libdir_spec_regex = re.compile('ln |\./configure[^#]*--libdir=([^\s]+)[^#]*')
 lib_package_regex = re.compile('^%package.*\Wlib')
 mklibname_regex = re.compile('%mklibname')
-ifarch_regex = re.compile('%ifn?arch')
+ifarch_regex = re.compile('%ifn?arch\s+')
 if_regex = re.compile('%if\s+')
-endif_regex = re.compile('%endif')
+endif_regex = re.compile('%endif\\b')
 biarch_package_regex = re.compile(DEFAULT_BIARCH_PACKAGES)
 hardcoded_lib_path_exceptions_regex = re.compile(Config.getOption('HardcodedLibPathExceptions', DEFAULT_HARDCODED_LIB_PATH_EXCEPTIONS))
 prereq_regex = re.compile('^PreReq(\(.*\))?:\s*(.+?)\s*$', re.IGNORECASE)
 buildprereq_regex = re.compile('^BuildPreReq:\s*(.+?)\s*$', re.IGNORECASE)
 use_utf8 = Config.getOption('UseUTF8', Config.USEUTF8_DEFAULT)
 macro_regex = re.compile('(%+)[{(]?(\w+)')
+libdir_regex = re.compile('%{?_lib(?:dir)?\}?\\b')
+comment_or_empty_regex = re.compile('^\s*(#|$)')
+defattr_regex = re.compile('^\s*%defattr\\b')
+attr_regex = re.compile('^\s*%attr\\b')
+section_regexs = dict(
+    ([x, re.compile('^%' + x + '(?:\s|$)')]
+     for x in ('build', 'changelog', 'check', 'clean', 'description', 'files',
+               'install', 'package', 'prep')))
 
 # Only check for /lib, /usr/lib, /usr/X11R6/lib
 # TODO: better handling of X libraries and modules.
@@ -60,8 +70,8 @@ scriptlet_requires_regex = re.compile('^(PreReq|Requires)\([^\)]*,', re.IGNORECA
 depscript_override_regex = re.compile('(^|\s)%(define|global)\s+__find_(requires|provides)\s')
 depgen_disable_regex = re.compile('(^|\s)%(define|global)\s+_use_internal_dependency_generator\s+0')
 
-indent_spaces_regex = re.compile(' {3}.*\S')
-indent_tabs_regex = re.compile('\t.*\S')
+# See https://bugzilla.redhat.com/488146 for details
+indent_spaces_regex = re.compile('( \t|(^|\t)([^\t]{8})*[^\t]{4}[^\t]?([^\t][^\t.!?]|[^\t]?[.!?] )  )')
 
 provides_regex = re.compile('^Provides(?:\([^\)]+\))?:\s*(.*)', re.IGNORECASE)
 obsoletes_regex = re.compile('^Obsoletes:\s*(.*)', re.IGNORECASE)
@@ -70,12 +80,26 @@ setup_q_regex = re.compile(' -[A-Za-z]*q')
 setup_t_regex = re.compile(' -[A-Za-z]*T')
 setup_ab_regex = re.compile(' -[A-Za-z]*[ab]')
 
+filelist_regex = re.compile('\s+-f\s+\S+')
+pkgname_regex = re.compile('\s+(?:-n\s+)?(\S+)')
+
 def deptokens(line):
     '''Parse provides/requires/conflicts/obsoletes line to dep token list.'''
     prco = []
     tmp = ''
     wantmore = 0
-    for tok in re.split('[\s,]+', line.strip()):
+    toks = re.split('[\s,]+', line.strip())
+
+    # Drop line continuation backslash in multiline macro definition, eg.
+    # [...] \
+    # Obsoletes: foo-%1 <= 1.0.0 \
+    # [...] \
+    # (yes, this is an ugly hack and we probably have other problems with
+    #  multiline macro definitions elsewhere...)
+    if toks[-1] == '\\':
+        del toks[-1]
+
+    for tok in toks:
         if len(tok) == 0:
             continue
         if len(tmp) == 0:
@@ -119,6 +143,7 @@ def contains_buildroot(line):
         return 1
     return 0
 
+
 class SpecCheck(AbstractCheck.AbstractCheck):
 
     def __init__(self):
@@ -129,23 +154,30 @@ class SpecCheck(AbstractCheck.AbstractCheck):
         if not pkg.isSource():
             return
 
+        wrong_spec = False
+
         # lookup spec file
-        files = pkg.files()
-        for f in files.keys():
-            if f.endswith('.spec'):
-                self._spec_file = pkg.dirName() + "/" + f
-                break
+        for fname, pkgfile in pkg.files().items():
+            if fname.endswith('.spec'):
+                self._spec_file = pkgfile.path
+                if fname == pkg.name + ".spec":
+                        wrong_spec = False
+                        break
+                else:
+                        wrong_spec = True
         if not self._spec_file:
             printError(pkg, "no-spec-file")
         else:
-            if f != pkg.name + ".spec":
-                printError(pkg, "invalid-spec-name", f)
+            if wrong_spec:
+                printError(pkg, "invalid-spec-name")
 
             # check content of spec file
-            spec_lines = Pkg.readlines(self._spec_file)
-            self.check_spec(pkg, spec_lines)
+            self.check_spec(pkg, self._spec_file)
 
-    def check_spec(self, pkg, spec_lines):
+    def check_spec(self, pkg, spec_file):
+        self._spec_file = spec_file
+        spec_only = isinstance(pkg, Pkg.FakePkg)
+        spec_lines = Pkg.readlines(spec_file)
         patches = {}
         applied_patches = []
         applied_patches_ifarch = []
@@ -157,48 +189,77 @@ class SpecCheck(AbstractCheck.AbstractCheck):
         lib = 0
         if_depth = 0
         ifarch_depth = -1
-        current_section = 'package'       
-        buildroot_clean={'clean':0 , 'install':0}
+        current_section = 'package'
+        buildroot_clean = {'clean': 0, 'install' : 0}
         depscript_override = 0
         depgen_disabled = 0
         indent_spaces = 0
         indent_tabs = 0
+        files_has_defattr = 0
         section = {}
-        for sec in ['description', 'prep', 'build', 'install', 'clean',
-                    'files', 'changelog', 'package', 'check']:
-            section[sec] = {
-                'count': 0,
-                're': re.compile('^%' + sec + '(?:\s|$)'),
-                }
+        # None == main package
+        current_package = None
+        package_noarch = {}
 
-        if self._spec_file:
-            if use_utf8 and not Pkg.is_utf8(self._spec_file):
+        is_utf8 = 0
+        if self._spec_file and use_utf8:
+            if Pkg.is_utf8(self._spec_file):
+                is_utf8 = 1
+            else:
                 printError(pkg, "non-utf8-spec-file", self._spec_file)
 
         # gather info from spec lines
 
         pkg.current_linenum = 0
+
+        nbsp = chr(0xA0)
+        if is_utf8:
+            nbsp = unichr(0xA0)
+
         for line in spec_lines:
 
             pkg.current_linenum += 1
+
+            if is_utf8:
+                line = unicode(line, "utf-8", "replace")
+
+            if line.find(nbsp) != -1:
+                printWarning(pkg, "non-break-space", "line %s" % pkg.current_linenum)
+
             section_marker = 0
-            for i in section.keys():
-                if section[i]['re'].search(line):
-                    current_section = i
+            for sec, regex in section_regexs.items():
+                res = regex.search(line)
+                if res:
+                    current_section = sec
                     section_marker = 1
-                    section[i]['count'] = section[i]['count'] + 1
+                    section[sec] = section.get(sec, 0) + 1
+                    if sec in ('package', 'files'):
+                        rest = filelist_regex.sub('', line[res.end()-1:])
+                        res = pkgname_regex.search(rest)
+                        if res:
+                            current_package = res.group(1)
+                        else:
+                            current_package = None
+                    break
 
             if section_marker:
+
+                if current_section == 'files':
+                    files_has_defattr = 0
+
+                if lib_package_regex.search(line):
+                    lib = 1
+
                 continue
 
             if current_section in ('prep', 'build'):
                 if contains_buildroot(line):
                     printWarning(pkg, 'rpm-buildroot-usage', '%' + current_section, line[:-1].strip())
 
-            if make_check_regexp.search(line) and current_section not in ('check', 'changelog', 'package', 'description'):
+            if make_check_regex.search(line) and current_section not in ('check', 'changelog', 'package', 'description'):
                 printWarning(pkg, 'make-check-outside-check-section', line[:-1])
 
-            if current_section in buildroot_clean.keys():
+            if current_section in buildroot_clean:
                 if contains_buildroot(line) and rm_regex.search(line):
                     buildroot_clean[current_section] = 1
 
@@ -217,96 +278,97 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                             printWarning(pkg, 'setup-not-quiet')
                     else:
                         printWarning(pkg, 'setup-not-quiet')
+                if current_section != 'prep':
+                    printWarning(pkg, 'setup-not-in-prep')
 
             if endif_regex.search(line):
                 if ifarch_depth == if_depth:
                     ifarch_depth = -1
                 if_depth = if_depth - 1
 
-            res=patch_regex.search(line)
+            res = applied_patch_regex.search(line)
             if res:
-                pnum = int(res.group(1) or 0)
-                patches[pnum] = res.group(2)
-            else:
-                res = applied_patch_regex.search(line)
-                if res:
-                    pnum = int(res.group(1) or res.group(2) or 0)
+                pnum = res.group(1) or 0
+                for tmp in applied_patch_p_regex.findall(line) or [pnum]:
+                    pnum = int(tmp);
                     applied_patches.append(pnum)
                     if ifarch_depth > 0:
                         applied_patches_ifarch.append(pnum)
-                elif not source_dir:
-                    res = source_dir_regex.search(line)
-                    if res:
-                        source_dir = 1
-                        printError(pkg, "use-of-RPM_SOURCE_DIR")
-
-            res=obsolete_tags_regex.search(line)
-            if res:
-                printWarning(pkg, "obsolete-tag", res.group(1))
+            elif not source_dir:
+                res = source_dir_regex.search(line)
+                if res:
+                    source_dir = 1
+                    printError(pkg, "use-of-RPM_SOURCE_DIR")
 
             if configure:
                 if configure_cmdline[-1] == "\\":
-                    configure_cmdline=configure_cmdline[:-1] + string.strip(line)
+                    configure_cmdline = configure_cmdline[:-1] + line.strip()
                 else:
-                    configure = 0
                     res = configure_libdir_spec_regex.search(configure_cmdline)
                     if not res:
-                        printError(pkg, "configure-without-libdir-spec")
+                        # Hack to get the correct (start of ./configure) line
+                        # number displayed:
+                        real_linenum = pkg.current_linenum
+                        pkg.current_linenum = configure
+                        printWarning(pkg, "configure-without-libdir-spec")
+                        pkg.current_linenum = real_linenum
                     elif res.group(1):
                         res = re.match(hardcoded_library_paths, res.group(1))
                         if res:
                             printError(pkg, "hardcoded-library-path", res.group(1), "in configure options")
+                    configure = 0
 
             if current_section != 'changelog' and configure_start_regex.search(line):
-                configure = 1
-                configure_cmdline = string.strip(line)
+                configure = pkg.current_linenum # store line where it started
+                configure_cmdline = line.strip()
 
             res = hardcoded_library_path_regex.search(line)
-            if current_section != 'changelog' and res and not (biarch_package_regex.match(pkg.name) or hardcoded_lib_path_exceptions_regex.search(string.lstrip(res.group(1)))):
-                printError(pkg, "hardcoded-library-path", "in", string.lstrip(res.group(1)))
-
-            res = buildroot_regex.search(line)
-            if res:
-                buildroot=1
-                if res.group(1).startswith('/'):
-                    printWarning(pkg, 'hardcoded-path-in-buildroot-tag', res.group(1))
-
-            res = packager_regex.search(line)
-            if res:
-                printWarning(pkg, 'hardcoded-packager-tag', res.group(1))
-            res=prefix_regex.search(line)
-            if res:
-                if res.group(1) == '%{_prefix}' or res.group(1) == '%_prefix':
-                    printWarning(pkg, 'redundant-prefix-tag')
-                else:
-                    printWarning(pkg, 'hardcoded-prefix-tag', res.group(1))
+            if current_section != 'changelog' and res and not (biarch_package_regex.match(pkg.name) or hardcoded_lib_path_exceptions_regex.search(res.group(1).lstrip())):
+                printError(pkg, "hardcoded-library-path", "in", res.group(1).lstrip())
 
             if mklibname_regex.search(line):
                 mklibname = 1
 
-            if lib_package_regex.search(line):
-                lib = 1
+            if current_section == 'package':
 
-            res = prereq_regex.search(line)
-            if res:
-                printWarning(pkg, 'prereq-use', res.group(2))
+                res = patch_regex.search(line)
+                if res:
+                    pnum = int(res.group(1) or 0)
+                    patches[pnum] = res.group(2)
 
-            res = buildprereq_regex.search(line)
-            if res:
-                printWarning(pkg, 'buildprereq-use', res.group(1))
+                res = obsolete_tags_regex.search(line)
+                if res:
+                    printWarning(pkg, "obsolete-tag", res.group(1))
 
-            if scriptlet_requires_regex.search(line) and current_section == 'package':
-                printError(pkg, 'broken-syntax-in-scriptlet-requires', string.strip(line))
+                res = buildroot_regex.search(line)
+                if res:
+                    buildroot = 1
+                    if res.group(1).startswith('/'):
+                        printWarning(pkg, 'hardcoded-path-in-buildroot-tag', res.group(1))
 
-            if current_section == 'changelog':
-                res = macro_regex.search(line)
-                if res and len(res.group(1)) % 2:
-                    printWarning(pkg, 'macro-in-%changelog', res.group(2))
-            else:
-                if not depscript_override:
-                    depscript_override = depscript_override_regex.search(line)
-                if not depgen_disabled:
-                    depgen_disabled = depgen_disable_regex.search(line)
+                res = packager_regex.search(line)
+                if res:
+                    printWarning(pkg, 'hardcoded-packager-tag', res.group(1))
+
+                res = prefix_regex.search(line)
+                if res:
+                    if not res.group(1).startswith('%'):
+                        printWarning(pkg, 'hardcoded-prefix-tag', res.group(1))
+
+                res = noarch_regex.search(line)
+                if res:
+                    package_noarch[current_package] = True
+
+                res = prereq_regex.search(line)
+                if res:
+                    printError(pkg, 'prereq-use', res.group(2))
+
+                res = buildprereq_regex.search(line)
+                if res:
+                    printError(pkg, 'buildprereq-use', res.group(1))
+
+                if scriptlet_requires_regex.search(line):
+                    printError(pkg, 'broken-syntax-in-scriptlet-requires', line.strip())
 
                 res = provides_regex.search(line)
                 if res:
@@ -318,11 +380,49 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                     for obs in unversioned(deptokens(res.group(1))):
                         printWarning(pkg, 'unversioned-explicit-obsoletes', obs)
 
-            if not indent_tabs and indent_tabs_regex.search(line):
+            if current_section == 'changelog':
+                res = macro_regex.search(line)
+                if res and len(res.group(1)) % 2:
+                    printWarning(pkg, 'macro-in-%changelog', res.group(2))
+            else:
+                if not depscript_override:
+                    depscript_override = depscript_override_regex.search(line)
+                if not depgen_disabled:
+                    depgen_disabled = depgen_disable_regex.search(line)
+
+            if current_section == 'files':
+
+                if not comment_or_empty_regex.search(line) and not \
+                   (ifarch_regex.search(line) or if_regex.search(line) or
+                    endif_regex.search(line)):
+                    if defattr_regex.search(line):
+                        files_has_defattr = 1;
+                    elif not (files_has_defattr or attr_regex.search(line)):
+                        printError(pkg, 'files-attr-not-set')
+
+                # TODO: check scriptlets for these too?
+                if package_noarch.get(current_package) or (current_package not in package_noarch and package_noarch.get(None)):
+                    res = libdir_regex.search(line)
+                    if res:
+                        pkgname = current_package
+                        if pkgname is None:
+                            pkgname = '(main package)'
+                        printWarning(pkg, 'libdir-macro-in-noarch-package',
+                                     pkgname, line.rstrip())
+
+            if not indent_tabs and line.find('\t') != -1:
                 indent_tabs = pkg.current_linenum
             if not indent_spaces and indent_spaces_regex.search(line):
                 indent_spaces = pkg.current_linenum
 
+            # If not checking spec file only, we're checking one inside a
+            # SRPM -> skip this check to avoid duplicate warnings (#167)
+            if spec_only and VALID_GROUPS and \
+                   line.lower().startswith("group:"):
+                group = line[6:].strip()
+                if group not in VALID_GROUPS:
+                    printWarning(pkg, 'non-standard-group', group)
+
         # No useful line number info beyond this point.
         pkg.current_linenum = None
 
@@ -334,11 +434,11 @@ class SpecCheck(AbstractCheck.AbstractCheck):
             printError(pkg, 'no-buildroot-tag')
 
         for sec in ('prep', 'build', 'install', 'clean'):
-            if not section[sec]['count']:
+            if not section.get(sec):
                 printWarning(pkg, 'no-%%%s-section' % sec)
         for sec in ('changelog',):
-            # prep, build, install, clean, check prevented by rpmbuild
-            if section[sec]['count'] > 1:
+            # prep, build, install, clean, check prevented by rpmbuild 4.4
+            if section.get(sec, 0) > 1:
                 printWarning(pkg, 'more-than-one-%%%s-section' % sec)
 
         if lib and not mklibname:
@@ -353,18 +453,27 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                          (indent_spaces, indent_tabs))
 
         # process gathered info
-        for p in patches.keys():
-            if p in applied_patches_ifarch:
-                printWarning(pkg, "%ifarch-applied-patch", "Patch%d:" % p, patches[p])
-            if p not in applied_patches:
-                printWarning(pkg, "patch-not-applied", "Patch%d:" % p, patches[p])
+        for pnum, pfile in patches.items():
+            if pnum in applied_patches_ifarch:
+                printWarning(pkg, "%ifarch-applied-patch", "Patch%d:" % pnum,
+                             pfile)
+            if pnum not in applied_patches:
+                printWarning(pkg, "patch-not-applied", "Patch%d:" % pnum,
+                             pfile)
+
+        out = Pkg.getstatusoutput(('env', 'LC_ALL=C', 'rpm', '-q',
+                                   '--qf=', '--specfile', self._spec_file))
+        for line in out[1].splitlines():
+            # No such file or dir hack: https://bugzilla.redhat.com/487855
+            if line.find("No such file or directory") < 0:
+                printError(pkg, 'specfile-error', line)
+
 
 # Create an object to enable the auto registration of the test
 check = SpecCheck()
 
 # Add information about checks
-if Config.info:
-    addDetails(
+addDetails(
 'no-spec-file',
 '''No spec file was specified in your RPM building. Please specify a valid
 SPEC file to build a valid RPM package.''',
@@ -405,17 +514,14 @@ as to use rebuilder's own defaults.''',
 '''The Prefix tag is hardcoded in your spec file. It should be removed, so as
 to allow package relocation.''',
 
-'redundant-prefix-tag',
-'''The Prefix tag is uselessly defined as %{_prefix} in your spec file. It
-should be removed, as it is redundant with rpm defaults.''',
-
 'hardcoded-library-path',
 '''A library path is hardcoded to one of the following paths: /lib,
 /usr/lib. It should be replaced by something like /%{_lib} or %{_libdir}.''',
 
 'configure-without-libdir-spec',
 '''A configure script is run without specifying the libdir. configure
-options must be augmented with something like --libdir=%{_libdir}.''',
+options must be augmented with something like --libdir=%{_libdir} whenever
+the script supports it.''',
 
 'no-%prep-section',
 '''The spec file does not contain a %prep section.  Even if some packages don't
@@ -468,6 +574,11 @@ versions of rpm.  One way to work around it is to split them into several ones,
 eg. replace "Requires(post,preun): foo" with "Requires(post): foo" and
 "Requires(preun): foo".''',
 
+'setup-not-in-prep',
+'''The %setup macro should only be used within the %prep section because it may
+not expand to anything outside of it and can break the build in unpredictable
+ways.''',
+
 'setup-not-quiet',
 '''Use the -q option to the %setup macro to avoid useless build output from
 unpacking the sources.''',
@@ -514,6 +625,33 @@ older, equal and newer versions of the obsoleted thing.  This may cause update
 problems, restrict future package/provides naming, and may match something it
 was originally not inteded to match -- make the Obsoletes versioned if
 possible.''',
+
+'libdir-macro-in-noarch-package',
+'''The %{_libdir} or %{_lib} macro was found in a noarch package in a section
+that gets included in binary packages.  This is most likely an error because
+these macros are expanded on the build host and their values vary between
+architectures, probably resulting in a package that does not work properly
+on all architectures at runtime. Investigate whether the package is really
+architecture independent or if some other dir/macro should be instead.''',
+
+'non-break-space',
+'''The spec file contains a non-break space, which looks like a regular space
+in some editors but can lead to obscure errors. It should be replaced by a
+regular space.''',
+
+'files-attr-not-set',
+'''A file or a directory entry in a %files section does not have attributes
+set which may result in security issues in the resulting binary package
+depending on the system where the package is built.  Add default attributes
+using %defattr before it in the %files section, or use per line %attr's.''',
+
+'non-standard-group',
+'''The value of the Group tag in the package is not valid.  Valid groups are:
+"%s".''' % '", "'.join(VALID_GROUPS),
+
+'specfile-error',
+'''This error occurred when rpmlint used rpm to query the specfile.  The error
+is output by rpm and the message should contain more information.''',
 )
 
 # SpecCheck.py ends here
