@@ -1,51 +1,30 @@
+# -*- coding: utf-8 -*-
 #############################################################################
 # File          : TagsCheck.py
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Tue Sep 28 00:03:24 1999
-# Version       : $Id: TagsCheck.py 1385 2007-11-27 20:28:06Z scop $
+# Version       : $Id: TagsCheck.py 1532 2009-01-30 22:01:50Z scop $
 # Purpose       : Check a package to see if some rpm tags are present
 #############################################################################
 
-from Filter import *
-import AbstractCheck
-import FilesCheck
-import Pkg 
-import rpm
-import string
+import os
 import re
-import os.path
+
+import rpm
+
+from Filter import addDetails, printError, printWarning
+import AbstractCheck
 import Config
-try:
-    from textwrap import fill # python >= 2.3
-except ImportError:
-    def fill(text, width=70): return text
-
-
-def get_default_valid_rpmgroups(filename=""):
-    """ Get the default rpm group from filename, or from rpm package if no
-    filename is given"""
-    groups = []
-    if not filename:
-        p = Pkg.InstalledPkg('rpm')
-        filename = filter(lambda x: x.endswith('/GROUPS'), p.files().keys())[0]
-    if filename and os.path.exists(filename):
-        fobj = open(filename)
-        try:
-            groups = fobj.read().strip().split('\n')
-        finally:
-            fobj.close()
-        if not 'Development/Debug' in groups:
-            groups.append('Development/Debug')
-            groups.sort()
-    return groups
+import FilesCheck
+import Pkg
 
 
 DEFAULT_VALID_LICENSES = (
     # OSI approved licenses, http://www.opensource.org/licenses/ (unversioned,
     # trailing "license" dropped based on fuzzy logic, and in well-known cases,
     # the abbreviation used instead of the full name, but list kept sorted by
-    # the full name).  Updated 2006-11-14.
+    # the full name).  Updated 2009-01-23.
     'Academic Free License',
     'Adaptive Public License',
     'Apache License',
@@ -65,21 +44,37 @@ DEFAULT_VALID_LICENSES = (
     'Entessa Public License',
     'Fair License',
     'Frameworx License',
-    'GPL', # GNU General Public License
-    'LGPL', # GNU Lesser General Public License
+    'GNU Affero GPLv3',
+    'GNU Affero GPLv3+',
+    'GPLv1',
+    'GPLv1+',
+    'GPLv2',
+    'GPLv2+',
+    'GPLv3',
+    'GPLv3+',
+    'LGPLv2',
+    'LGPLv2+',
+    'LGPLv3',
+    'LGPLv3+',
     'Historical Permission Notice and Disclaimer',
     'IBM Public License',
     'Intel Open Source License',
+    'ISC License',
     'Jabber Open Source License',
     'Lucent Public License',
+    'Microsoft Public License',
+    'Microsoft Reciprocal License',
     'MIT',
     'CVW License', # MITRE Collaborative Virtual Workspace License
     'Motosoto License',
     'MPL', # Mozilla Public License
+    'Multics License',
     'NASA Open Source Agreement',
     'Naumen Public License',
     'Nethack General Public License',
     'Nokia Open Source License',
+    'Non-profit Open Software License',
+    'NTP License',
     'OCLC Research Public License',
     'Open Group Test Suite License',
     'Open Software License',
@@ -385,44 +380,45 @@ BAD_WORDS = {
     'xwindows': 'X'
     }
 
-DEFAULT_INVALID_REQUIRES=('^is$', '^not$', '^owned$', '^by$', '^any$', '^package$', '^libsafe\.so\.')
+DEFAULT_INVALID_REQUIRES = ('^is$', '^not$', '^owned$', '^by$', '^any$', '^package$', '^libsafe\.so\.')
 
-VALID_GROUPS=Config.getOption('ValidGroups', get_default_valid_rpmgroups())
-VALID_LICENSES=Config.getOption('ValidLicenses', DEFAULT_VALID_LICENSES)
-INVALID_REQUIRES=map(lambda x: re.compile(x), Config.getOption('InvalidRequires', DEFAULT_INVALID_REQUIRES))
-packager_regex=re.compile(Config.getOption('Packager'))
-basename_regex=re.compile('/?([^/]+)$')
-changelog_version_regex=re.compile('[^>]([^ >]+)\s*$')
-changelog_text_version_regex=re.compile('^\s*-\s*((\d+:)?[\w\.]+-[\w\.]+)')
-release_ext=Config.getOption('ReleaseExtension')
-extension_regex=release_ext and re.compile(release_ext + '$')
-use_version_in_changelog=Config.getOption('UseVersionInChangelog', 1)
-devel_number_regex=re.compile('(.*?)([0-9.]+)(_[0-9.]+)?-devel')
-lib_devel_number_regex=re.compile('^lib(.*?)([0-9.]+)(_[0-9.]+)?-devel')
-url_regex=re.compile('^(ftp|http|https)://')
-invalid_url_regex=re.compile(Config.getOption('InvalidURL'), re.IGNORECASE)
-lib_regex=re.compile('^lib.*?(\.so.*)?$')
-leading_space_regex=re.compile('^\s+')
-invalid_version_regex=re.compile('([0-9](?:rc|alpha|beta|pre).*)', re.IGNORECASE)
+VALID_GROUPS = Config.getOption('ValidGroups', Pkg.get_default_valid_rpmgroups())
+VALID_LICENSES = Config.getOption('ValidLicenses', DEFAULT_VALID_LICENSES)
+INVALID_REQUIRES = map(re.compile, Config.getOption('InvalidRequires', DEFAULT_INVALID_REQUIRES))
+packager_regex = re.compile(Config.getOption('Packager'))
+changelog_version_regex = re.compile('[^>]([^ >]+)\s*$')
+changelog_text_version_regex = re.compile('^\s*-\s*((\d+:)?[\w\.]+-[\w\.]+)')
+release_ext = Config.getOption('ReleaseExtension')
+extension_regex = release_ext and re.compile(release_ext)
+use_version_in_changelog = Config.getOption('UseVersionInChangelog', 1)
+devel_number_regex = re.compile('(.*?)([0-9.]+)(_[0-9.]+)?-devel')
+lib_devel_number_regex = re.compile('^lib(.*?)([0-9.]+)(_[0-9.]+)?-devel')
+url_regex = re.compile('^(ftp|http|https)://')
+invalid_url_regex = re.compile(Config.getOption('InvalidURL'), re.IGNORECASE)
+lib_package_regex = re.compile('(?:^(?:compat-)?lib.*?(\.so.*)?|libs?[\d-]*)$', re.IGNORECASE)
+leading_space_regex = re.compile('^\s+')
+license_regex = re.compile('\(([^)]+)\)|\s(?:and|or)\s')
+invalid_version_regex = re.compile('([0-9](?:rc|alpha|beta|pre).*)', re.IGNORECASE)
 # () are here for grouping purpose in the regexp
-forbidden_words_regex=re.compile('(' + Config.getOption('ForbiddenWords') + ')', re.IGNORECASE)
-valid_buildhost_regex=re.compile(Config.getOption('ValidBuildHost'))
-epoch_regex=re.compile('^[0-9]+:')
-use_epoch=Config.getOption('UseEpoch', 0)
-use_utf8=Config.getOption('UseUTF8', Config.USEUTF8_DEFAULT)
-max_line_len=79
-tag_regex=re.compile('^((?:Auto(?:Req|Prov|ReqProv)|Build(?:Arch(?:itectures)?|Root)|(?:Build)?Conflicts|(?:Build)?(?:Pre)?Requires|Copyright|(?:CVS|SVN)Id|Dist(?:ribution|Tag|URL)|DocDir|Epoch|Exclu(?:de|sive)(?:Arch|OS)|Group|Icon|License|Name|No(?:Patch|Source)|Obsoletes|Packager|Patch\d*|Prefix(?:es)?|Provides|Release|RHNPlatform|Serial|Source\d*|Summary|URL|Vendor|Version)(?:\([^)]+\))?:)\s*\S', re.IGNORECASE)
+forbidden_words_regex = re.compile('(' + Config.getOption('ForbiddenWords') + ')', re.IGNORECASE)
+valid_buildhost_regex = re.compile(Config.getOption('ValidBuildHost'))
+epoch_regex = re.compile('^[0-9]+:')
+use_epoch = Config.getOption('UseEpoch', 0)
+use_utf8 = Config.getOption('UseUTF8', Config.USEUTF8_DEFAULT)
+max_line_len = 79
+tag_regex = re.compile('^((?:Auto(?:Req|Prov|ReqProv)|Build(?:Arch(?:itectures)?|Root)|(?:Build)?Conflicts|(?:Build)?(?:Pre)?Requires|Copyright|(?:CVS|SVN)Id|Dist(?:ribution|Tag|URL)|DocDir|(?:Build)?Enhances|Epoch|Exclu(?:de|sive)(?:Arch|OS)|Group|Icon|License|Name|No(?:Patch|Source)|Obsoletes|Packager|Patch\d*|Prefix(?:es)?|Provides|(?:Build)?Recommends|Release|RHNPlatform|Serial|Source\d*|(?:Build)?Suggests|Summary|(?:Build)?Supplements|URL|Vendor|Version)(?:\([^)]+\))?:)\s*\S', re.IGNORECASE)
+punct = '.,:;!?'
 
 def spell_check(pkg, str, tagname):
-    for seq in string.split(str, ' '):
-        for word in re.split('[^a-z]+', string.lower(seq)):
+    for seq in str.split():
+        for word in re.split('[^a-z]+', seq.lower()):
             if len(word) > 0:
                 try:
                     if word[0] == '\'':
-                        word=word[1:]
+                        word = word[1:]
                     if word[-1] == '\'':
-                        word=word[:-1]
-                    correct=BAD_WORDS[word]
+                        word = word[:-1]
+                    correct = BAD_WORDS[word]
                     printWarning(pkg, 'spelling-error-in-' + tagname, word, correct)
                 except KeyError:
                     pass
@@ -434,27 +430,27 @@ class TagsCheck(AbstractCheck.AbstractCheck):
 
     def check(self, pkg):
 
-        packager=pkg[rpm.RPMTAG_PACKAGER]
+        packager = pkg[rpm.RPMTAG_PACKAGER]
         if not packager:
             printError(pkg, 'no-packager-tag')
         elif Config.getOption('Packager') and not packager_regex.search(packager):
             printWarning(pkg, 'invalid-packager', packager)
 
-        version=pkg[rpm.RPMTAG_VERSION]
+        version = pkg[rpm.RPMTAG_VERSION]
         if not version:
             printError(pkg, 'no-version-tag')
         else:
-            res=invalid_version_regex.search(version)
+            res = invalid_version_regex.search(version)
             if res:
                 printError(pkg, 'invalid-version', version)
 
-        release=pkg[rpm.RPMTAG_RELEASE]
+        release = pkg[rpm.RPMTAG_RELEASE]
         if not release:
             printError(pkg, 'no-release-tag')
         elif release_ext and not extension_regex.search(release):
             printWarning(pkg, 'not-standard-release-extension', release)
 
-        epoch=pkg[rpm.RPMTAG_EPOCH]
+        epoch = pkg[rpm.RPMTAG_EPOCH]
         if epoch is None:
             if use_epoch:
                 printError(pkg, 'no-epoch-tag')
@@ -473,11 +469,11 @@ class TagsCheck(AbstractCheck.AbstractCheck):
                 if p[1] and not epoch_regex.search(p[1]):
                     printWarning(pkg, 'no-epoch-in-provides', p[0] + ' ' + p[1])
 
-        name=pkg.name
-        deps=pkg.requires() + pkg.prereq()
-        devel_depend=0
-        is_devel=FilesCheck.devel_regex.search(name)
-        is_source=pkg.isSource()
+        name = pkg.name
+        deps = pkg.requires() + pkg.prereq()
+        devel_depend = 0
+        is_devel = FilesCheck.devel_regex.search(name)
+        is_source = pkg.isSource()
         for d in deps:
             if use_epoch and d[1] and d[0][0:7] != 'rpmlib(' and not epoch_regex.search(d[1]):
                 printWarning(pkg, 'no-epoch-in-dependency', d[0] + ' ' + d[1])
@@ -491,59 +487,60 @@ class TagsCheck(AbstractCheck.AbstractCheck):
             if not devel_depend and not is_devel and not is_source:
                 if FilesCheck.devel_regex.search(d[0]):
                     printError(pkg, 'devel-dependency', d[0])
-                    devel_depend=1
+                    devel_depend = 1
             if is_source and lib_devel_number_regex.search(d[0]):
                 printError(pkg, 'invalid-build-requires', d[0])
             if not is_source and not is_devel:
-                res=lib_regex.search(d[0])
+                res = lib_package_regex.search(d[0])
                 if res and not res.group(1) and not d[1]:
                     printError(pkg, 'explicit-lib-dependency', d[0])
-            if d[2] == rpm.RPMSENSE_EQUAL and string.find(d[1], '-') != -1:
+            if d[2] == rpm.RPMSENSE_EQUAL and d[1].find('-') != -1:
                 printWarning(pkg, 'requires-on-release', d[0], d[1])
-            if string.find(d[1], '%') != -1:
+            if d[1].find('%') != -1:
                 printError(pkg, 'percent-in-dependency', d[0], d[1])
 
         if not name:
             printError(pkg, 'no-name-tag')
         else:
             if is_devel and not is_source:
-                base=is_devel.group(1)
-                dep=None
-                has_so=0
-                for f in pkg.files().keys():
-                    if f.endswith('.so'):
-                        has_so=1
+                base = is_devel.group(1)
+                dep = None
+                has_so = 0
+                for fname in pkg.files():
+                    if fname.endswith('.so'):
+                        has_so = 1
                         break
                 if has_so:
+                    base_or_libs = base + '/' + base + '-libs/lib' + base
                     for d in deps:
-                        if d[0] == base:
-                            dep=d
+                        if d[0] == base or d[0] == base + '-libs' or d[0] == 'lib' + base:
+                            dep = d
                             break
                     if not dep:
-                        printWarning(pkg, 'no-dependency-on', base)
+                        printWarning(pkg, 'no-dependency-on', base_or_libs)
                     elif version:
                         if epoch is not None: # regardless of use_epoch
-                            expected=str(epoch) + ":" + version
+                            expected = str(epoch) + ":" + version
                         else:
-                            expected=version
+                            expected = version
                         if dep[1][:len(expected)] != expected:
                             if dep[1] != '':
-                                printWarning(pkg, 'incoherent-version-dependency-on', base, dep[1], expected)
+                                printWarning(pkg, 'incoherent-version-dependency-on', base_or_libs, dep[1], expected)
                             else:
-                                printWarning(pkg, 'no-version-dependency-on', base, expected)
-                    res=devel_number_regex.search(name)
+                                printWarning(pkg, 'no-version-dependency-on', base_or_libs, expected)
+                    res = devel_number_regex.search(name)
                     if not res:
                         printWarning(pkg, 'no-major-in-name', name)
                     else:
                         if res.group(3):
-                            prov=res.group(1) + res.group(2) + '-devel'
+                            prov = res.group(1) + res.group(2) + '-devel'
                         else:
-                            prov=res.group(1) + '-devel'
+                            prov = res.group(1) + '-devel'
 
-                        if not prov in map(lambda x: x[0], pkg.provides()):
+                        if prov not in (x[0] for x in pkg.provides()):
                             printWarning(pkg, 'no-provides', prov)
 
-        summary=pkg[rpm.RPMTAG_SUMMARY]
+        summary = pkg[rpm.RPMTAG_SUMMARY]
         if not summary:
             printError(pkg, 'no-summary-tag')
         else:
@@ -551,7 +548,7 @@ class TagsCheck(AbstractCheck.AbstractCheck):
             if use_utf8:
                 utf8summary = Pkg.to_utf8(summary).decode('utf-8')
             spell_check(pkg, summary, 'summary')
-            if string.find(summary, '\n') != -1:
+            if summary.find('\n') != -1:
                 printError(pkg, 'summary-on-multiple-lines')
             if summary[0] != summary[0].upper():
                 printWarning(pkg, 'summary-not-capitalized', summary)
@@ -561,24 +558,31 @@ class TagsCheck(AbstractCheck.AbstractCheck):
                 printError(pkg, 'summary-too-long', summary)
             if leading_space_regex.search(summary):
                 printError(pkg, 'summary-has-leading-spaces', summary)
-            res=forbidden_words_regex.search(summary)
+            res = forbidden_words_regex.search(summary)
             if res and Config.getOption('ForbiddenWords'):
                 printWarning(pkg, 'summary-use-invalid-word', res.group(1))
+            if name:
+                sepchars = '[\s' + punct + ']'
+                res = re.search('(?:^|\s)(%s)(?:%s|$)' %
+                                (re.escape(name), sepchars),
+                                summary, re.IGNORECASE | re.UNICODE)
+                if res:
+                    printWarning(pkg, 'name-repeated-in-summary', res.group(1))
             if use_utf8 and not Pkg.is_utf8_str(summary):
                 printError(pkg, 'tag-not-utf8', 'Summary')
 
-        description=pkg[rpm.RPMTAG_DESCRIPTION]
+        description = pkg[rpm.RPMTAG_DESCRIPTION]
         if not description:
             printError(pkg, 'no-description-tag')
         else:
             spell_check(pkg, description, 'description')
-            for l in string.split(description, "\n"):
+            for l in description.splitlines():
                 utf8l = l
                 if use_utf8:
                     utf8l = Pkg.to_utf8(l).decode('utf-8')
                 if len(utf8l) > max_line_len:
                     printError(pkg, 'description-line-too-long', l)
-                res=forbidden_words_regex.search(l)
+                res = forbidden_words_regex.search(l)
                 if res and Config.getOption('ForbiddenWords'):
                     printWarning(pkg, 'description-use-invalid-word', res.group(1))
                 res = tag_regex.search(l)
@@ -587,65 +591,75 @@ class TagsCheck(AbstractCheck.AbstractCheck):
             if use_utf8 and not Pkg.is_utf8_str(description):
                 printError(pkg, 'tag-not-utf8', '%description')
 
-        group=pkg[rpm.RPMTAG_GROUP]
+        group = pkg[rpm.RPMTAG_GROUP]
         if not group:
             printError(pkg, 'no-group-tag')
         else:
             if VALID_GROUPS and group not in VALID_GROUPS:
                 printWarning(pkg, 'non-standard-group', group)
 
-        buildhost=pkg[rpm.RPMTAG_BUILDHOST]
+        buildhost = pkg[rpm.RPMTAG_BUILDHOST]
         if not buildhost:
             printError(pkg, 'no-buildhost-tag')
         else:
             if Config.getOption('ValidBuildHost') and not valid_buildhost_regex.search(buildhost):
                 printWarning(pkg, 'invalid-buildhost', buildhost)
 
-        changelog=pkg[rpm.RPMTAG_CHANGELOGNAME]
+        changelog = pkg[rpm.RPMTAG_CHANGELOGNAME]
         if not changelog:
             printError(pkg, 'no-changelogname-tag')
         else:
-            clt=pkg[rpm.RPMTAG_CHANGELOGTEXT]
-            if use_version_in_changelog and not pkg.isSource():
-                ret=changelog_version_regex.search(changelog[0])
+            clt = pkg[rpm.RPMTAG_CHANGELOGTEXT]
+            if use_version_in_changelog:
+                ret = changelog_version_regex.search(changelog[0])
                 if not ret and clt:
                     # we also allow the version specified as the first
                     # thing on the first line of the text
-                    ret=changelog_text_version_regex.search(clt[0])
+                    ret = changelog_text_version_regex.search(clt[0])
                 if not ret:
                     printWarning(pkg, 'no-version-in-last-changelog')
                 elif version and release:
                     srpm = pkg[rpm.RPMTAG_SOURCERPM] or ''
                     # only check when source name correspond to name
                     if srpm[0:-8] == '%s-%s-%s' % (name, version, release):
-                        expected=version + '-' + release
+                        expected = [version + '-' + release]
                         if epoch is not None: # regardless of use_epoch
-                            expected=str(epoch) + ':' + expected
-                        if expected != ret.group(1):
+                            expected[0] = str(epoch) + ':' + expected[0]
+                        # Allow EVR in changelog without release extension,
+                        # the extension is often a macro or otherwise dynamic.
+                        if release_ext:
+                            expected.append(extension_regex.sub('', expected[0]))
+                        if ret.group(1) not in expected:
+                            if len(expected) == 1: expected = expected[0]
                             printWarning(pkg, 'incoherent-version-in-changelog', ret.group(1), expected)
 
-            if clt: changelog=changelog + clt
+            if clt: changelog = changelog + clt
             if use_utf8 and not Pkg.is_utf8_str(' '.join(changelog)):
                 printError(pkg, 'tag-not-utf8', '%changelog')
 
-#         provides=pkg.provides()
+#         provides = pkg.provides()
 #         for (provide_name, provide_version, provide_flags) in provides:
 #             if name == provide_name:
 #                 printWarning(pkg, 'package-provides-itself')
 #                 break
 
+        def split_license(license):
+            return (x.strip() for x in
+                    (l for l in license_regex.split(license) if l))
+
         rpm_license = pkg[rpm.RPMTAG_LICENSE]
         if not rpm_license:
             printError(pkg, 'no-license')
         else:
             if rpm_license not in VALID_LICENSES:
-                licenses = re.split('(?:[- ]like|/|ish|[- ]style|[- ]Style|and|or|&|\s|-)+', rpm_license)
-                for l in licenses:
-                    if l != '' and not l in VALID_LICENSES:
-                        printWarning(pkg, 'invalid-license', rpm_license)
-                        break
+                for l1 in split_license(rpm_license):
+                    if l1 in VALID_LICENSES:
+                        continue
+                    for l2 in split_license(l1):
+                        if l2 not in VALID_LICENSES:
+                            printWarning(pkg, 'invalid-license', l2)
 
-        url=pkg[rpm.RPMTAG_URL]
+        url = pkg[rpm.RPMTAG_URL]
         if url and url != 'none':
             if not url_regex.search(url):
                 printWarning(pkg, 'invalid-url', url)
@@ -654,52 +668,54 @@ class TagsCheck(AbstractCheck.AbstractCheck):
         else:
             printWarning(pkg, 'no-url-tag')
 
-        obs=map(lambda x: x[0], pkg.obsoletes())
-        provs=map(lambda x: x[0], pkg.provides())
-        if pkg.name in obs:
+        obs_names = [x[0] for x in pkg.obsoletes()]
+        prov_names = [x[0] for x in pkg.provides()]
+
+        if pkg.name in obs_names:
             printError(pkg, 'obsolete-on-name')
-        for o in obs:
-            if not o in provs:
+        for o in obs_names:
+            if not o in prov_names:
                 printWarning(pkg, 'obsolete-not-provided', o)
         for o in pkg.obsoletes():
-            if string.find(o[1], '%') != -1:
+            if o[1].find('%') != -1:
                 printError(pkg, 'percent-in-obsoletes', o[0], o[1])
 
-        useless_provides=[]
-        for p in provs:
-            if provs.count(p) != 1:
+        # TODO: should take versions, <, <=, =, >=, > into account here
+        #       https://bugzilla.redhat.com/460872
+        useless_provides = []
+        for p in prov_names:
+            if prov_names.count(p) != 1:
                 if p not in useless_provides:
                     useless_provides.append(p)
         for p in useless_provides:
-            printError(pkg, 'useless-explicit-provides',p)
+            printError(pkg, 'useless-provides', p)
 
         for p in pkg.provides():
-            if string.find(p[1], '%') != -1:
+            if p[1].find('%') != -1:
                 printError(pkg, 'percent-in-provides', p[0], p[1])
 
         for c in pkg.conflicts():
-            if string.find(c[1], '%') != -1:
+            if c[1].find('%') != -1:
                 printError(pkg, 'percent-in-conflicts', c[0], c[1])
 
-        expected='%s-%s-%s.%s.rpm' % (name, version, release, pkg.arch)
-        basename=string.split(pkg.filename, '/')[-1]
+        expected = '%s-%s-%s.%s.rpm' % (name, version, release, pkg.arch)
+        basename = os.path.basename(pkg.filename)
         if basename != expected:
             printWarning(pkg, 'non-coherent-filename', basename, expected)
 
 # Create an object to enable the auto registration of the test
-check=TagsCheck()
+check = TagsCheck()
 
 # Add information about checks
-if Config.info:
-    addDetails(
+addDetails(
 'summary-too-long',
 'The "Summary:" must not exceed %d characters.' % max_line_len,
 
 'invalid-version',
 '''The version string must not contain the pre, alpha, beta or rc suffixes
 because when the final version will be out, you will have to use an Epoch tag
-to make the package upgradable. Instead put it in the release tag like
-0.alpha8.1''' + release_ext + '.',
+to make the package upgradable. Instead put it in the release tag, prefixed
+with something you have control over.''',
 
 'spelling-error-in-description',
 '''You made a misspelling in the Description. Please double-check.''',
@@ -724,7 +740,7 @@ the Version tag.''',
 the Release tag.''',
 
 'not-standard-release-extension',
-'Your release number must end with ' + release_ext + ' and must be valid.',
+'Your release tag must match the regular expression ' + release_ext + '.',
 
 'no-name-tag',
 '''There is no Name tag in your package. You have to specify a name using the
@@ -793,7 +809,7 @@ in your spec file using the Group tag.''',
 
 'non-standard-group',
 '''The value of the Group tag in the package is not valid.  Valid groups are:
-%s''' % fill('"' + '", "'.join(VALID_GROUPS) + '".', 78),
+"%s".''' % '", "'.join(VALID_GROUPS),
 
 'no-changelogname-tag',
 '''There is no %changelog tag in your spec file. To insert it, just insert a
@@ -814,9 +830,7 @@ your specfile.''',
 
 'invalid-license',
 '''The value of the License tag was not recognized.  Known values are:
-%s
-If the license is close to an existing one, you can use '<license> style'.''' \
-% fill('"' + '", "'.join(VALID_LICENSES) + '".', 78),
+"%s".''' % '", "'.join(VALID_LICENSES),
 
 'invalid-url',
 '''Your URL is not valid. It must begin with http, https or ftp and must no
@@ -824,9 +838,9 @@ longer contain the word mandrake.''',
 
 'obsolete-not-provided',
 '''If a package is obsoleted by a compatible replacement, the obsoleted package
-must also be provided in order to provide clean upgrade paths and not cause
-unnecessary dependency breakage.  If the obsoleting package is not a compatible
-replacement for the old one, leave out the provides.''',
+should also be provided in order to not cause unnecessary dependency breakage.
+If the obsoleting package is not a compatible replacement for the old one,
+leave out the Provides.''',
 
 'invalid-dependency',
 '''An invalid dependency has been detected. It usually means that the build of
@@ -864,7 +878,7 @@ This BuildRequires dependency will not be resolved on lib64 platforms
 '''You must let rpm find the library dependencies by itself. Do not put unneeded
 explicit Requires: tags.''',
 
-'useless-explicit-provides',
+'useless-provides',
 '''This package provides 2 times the same capacity. It should only provide it
 once.''',
 
@@ -895,6 +909,11 @@ It could be an unexpanded macro, please double check.''',
 
 'no-url-tag',
 '''The URL tag is missing.''',
+
+'name-repeated-in-summary',
+'''The name of the package is repeated in its summary.  This is often redundant
+information and looks silly in various programs' output.  Make the summary
+brief and to the point without including redundant information in it.''',
 )
 
 # TagsCheck.py ends here
